# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from phonopy.structure.cells import get_reduced_bases

def get_dynamical_matrix(fc2,
                         supercell,
                         primitive,
                         nac_params=None,
                         frequency_scale_factor=None,
                         decimals=None,
                         symprec=1e-5):
    if frequency_scale_factor is None:
        _fc2 = fc2
    else:
        _fc2 = fc2 * frequency_scale_factor ** 2

    if nac_params is None:
        dm = DynamicalMatrix(
            supercell,
            primitive,
            _fc2,
            decimals=decimals,
            symprec=symprec)
    else:
        dm = DynamicalMatrixNAC(
            supercell,
            primitive,
            _fc2,
            decimals=decimals,
            symprec=symprec)
        dm.set_nac_params(nac_params)
    return dm

class DynamicalMatrix(object):
    """Dynamical matrix class
    
    When prmitive and supercell lattices are L_p and L_s, respectively,
    frame F is defined by
    L_p = dot(F, L_s), then L_s = dot(F^-1, L_p).
    where lattice matrix is defined by axies a,b,c in Cartesian:
        [ a1 a2 a3 ]
    L = [ b1 b2 b3 ]
        [ c1 c2 c3 ]
    
    Phase difference in primitive cell unit
    between atoms 1 and 2 in supercell is calculated by, e.g.,
    1j * dot((x_s(2) - x_s(1)), F^-1) * 2pi
    where x_s is reduced atomic coordinate in supercell unit.
    """

    def __init__(self,
                 supercell,
                 primitive,
                 force_constants,
                 decimals=None,
                 symprec=1e-5):
        self._scell = supercell
        self._pcell = primitive
        self._force_constants = np.array(force_constants,
                                         dtype='double', order='C')
        self._decimals = decimals
        self._symprec = symprec

        self._p2s_map = primitive.get_primitive_to_supercell_map()
        self._s2p_map = primitive.get_supercell_to_primitive_map()
        p2p_map = primitive.get_primitive_to_primitive_map()
        self._p2p_map = [p2p_map[self._s2p_map[i]]
                         for i in range(len(self._s2p_map))]
        self._smallest_vectors, self._multiplicity = \
            get_smallest_vectors(supercell, primitive, symprec)
        self._mass = self._pcell.get_masses()
        # Non analytical term correction
        self._nac = False

    def is_nac(self):
        return self._nac

    def get_dimension(self):
        return self._pcell.get_number_of_atoms() * 3

    def get_decimals(self):
        return self._decimals
    
    def get_supercell(self):
        return self._scell

    def get_primitive(self):
        return self._pcell

    def get_force_constants(self):
        return self._force_constants

    def get_shortest_vectors(self):
        return self._smallest_vectors, self._multiplicity

    def get_primitive_to_supercell_map(self):
        return self._p2s_map

    def get_supercell_to_primitive_map(self):
        return self._s2p_map

    def get_dynamical_matrix(self):
        dm = self._dynamical_matrix

        if self._decimals is None:
            return dm
        else:
            return dm.round(decimals=self._decimals)

    def set_dynamical_matrix(self, q, verbose=False):
        try:
            import phonopy._phonopy as phonoc
            self._set_c_dynamical_matrix(q)
        except ImportError:
            self._set_py_dynamical_matrix(q, verbose=verbose)

    def _set_py_dynamical_matrix(self,
                                 q,
                                 verbose=False):
        fc = self._force_constants
        vecs = self._smallest_vectors
        multiplicity = self._multiplicity
        num_atom = len(self._p2s_map)
        dm = np.zeros((3 * num_atom, 3 * num_atom), dtype=complex)

        for i, s_i in enumerate(self._p2s_map):
            for j, s_j in enumerate(self._p2s_map):
                mass = np.sqrt(self._mass[i] * self._mass[j])
                dm_local = np.zeros((3, 3), dtype=complex)
                # Sum in lattice points                
                for k in range(self._scell.get_number_of_atoms()): 
                    if s_j == self._s2p_map[k]:
                        multi = multiplicity[k][i]
                        phase = []
                        for l in range(multi):
                            vec = vecs[k][i][l]
                            phase.append(np.vdot(vec, q) * 2j * np.pi)
                        phase_factor = np.exp(phase).sum()
                        dm_local += fc[s_i, k] * phase_factor / mass / multi

                dm[(i*3):(i*3+3), (j*3):(j*3+3)] += dm_local

        # Impose Hermisian condition
        self._dynamical_matrix = (dm + dm.conj().transpose()) / 2 

        if verbose:
            self._dynamical_matrix_log()

    def _dynamical_matrix_log(self):
        dm = self._dynamical_matrix
        for i in range(dm.shape[0] // 3):
            for j in range(dm.shape[0] // 3):
                dm_local = dm[(i*3):(i*3+3), (j*3):(j*3+3)]
                for vec in dm_local:
                    re = vec.real
                    im = vec.imag
                    print("dynamical matrix(%3d - %3d) "
                          "%10.5f %10.5f %10.5f %10.5f %10.5f %10.5f" % 
                          (i+1, j+1, re[0], im[0], re[1], im[1], re[2], im[2]))
                print('')

    def _smallest_vectors_log(self):
        r = self._smallest_vectors
        m = self._multiplicity

        print("#%4s %4s %4s %4s %4s %10s" % 
              ("p_i", "p_j", "s_i", "s_j", "mult", "dist"))
        for p_i, s_i in enumerate(self._p2s_map): # run in primitive
            for s_j in range(r.shape[0]): # run in supercell
                for tmp_p_j, tmp_s_j in enumerate(self._p2s_map):
                    if self._s2p_map[s_j] == tmp_s_j:
                        p_j = tmp_p_j
                for k in range(m[s_j][p_i]):
                    print(" %4d %4d %4d %4d %4d %10.5f" %
                          (p_i+1, p_j+1, s_i+1, s_j+1, m[s_j][p_i],
                           np.linalg.norm(np.dot(r[s_j][p_i][k],
                                                 self._pcell.get_cell()))))

    def _set_c_dynamical_matrix(self, q):
        import phonopy._phonopy as phonoc

        fc = self._force_constants
        vectors = self._smallest_vectors
        mass = self._pcell.get_masses()
        multiplicity = self._multiplicity
        size_prim = len(mass)
        itemsize = self._force_constants.itemsize
        dm = np.zeros((size_prim * 3, size_prim * 3),
                      dtype=("c%d" % (itemsize * 2)))
        phonoc.dynamical_matrix(dm.view(dtype='double'),
                                fc,
                                np.array(q, dtype='double'),
                                vectors,
                                multiplicity,
                                mass,
                                self._s2p_map,
                                self._p2s_map)
        self._dynamical_matrix = dm

# Non analytical term correction (NAC)
# Call this when NAC is required instead of DynamicalMatrix
class DynamicalMatrixNAC(DynamicalMatrix):
    def __init__(self,
                 supercell,
                 primitive,
                 force_constants,
                 nac_params=None,
                 decimals=None,
                 symprec=1e-5):

        DynamicalMatrix.__init__(self,
                                 supercell,
                                 primitive,
                                 force_constants,
                                 decimals=decimals,
                                 symprec=1e-5)
        self._bare_force_constants = self._force_constants.copy()

        self._nac = True
        if nac_params is not None:
            self.set_nac_params(nac_params)

    def get_born_effective_charges(self):
        return self._born

    def get_nac_factor(self):
        return self._unit_conversion * 4.0 * np.pi / self._pcell.get_volume()

    def get_dielectric_constant(self):
        return self._dielectric
    
    def set_nac_params(self, nac_params):
        self._born = np.array(nac_params['born'], dtype='double', order='C')
        self._unit_conversion = nac_params['factor']
        self._dielectric = np.array(nac_params['dielectric'],
                                    dtype='double', order='C')

    def set_dynamical_matrix(self, q_red, q_direction=None, verbose=False):
        num_atom = self._pcell.get_number_of_atoms()

        if q_direction is None:
            q = np.dot(q_red, np.linalg.inv(self._pcell.get_cell()).T)
        else:
            q = np.dot(q_direction, np.linalg.inv(self._pcell.get_cell()).T)

        if (q_direction is None and np.abs(q).sum() < self._symprec) or \
                ((q_direction is not None) and
                 np.abs(q_direction).sum() < self._symprec):
            self._force_constants = self._bare_force_constants.copy()
            DynamicalMatrix.set_dynamical_matrix(self, q_red, verbose)
            return False
    
        volume = self._pcell.get_volume()
        constant = (self._unit_conversion * 4.0 * np.pi / volume
                    / np.dot(q, np.dot(self._dielectric, q)))

        # Wang method (J. Phys.: Condens. Matter 22 (2010) 202201)
        import phonopy._phonopy as phonoc
        try:
            import phonopy._phonopy as phonoc
            self._set_c_nac_dynamical_matrix(q_red, q, constant)
        except ImportError:
            fc = self._bare_force_constants.copy()
            nac_q = np.zeros((num_atom, num_atom, 3, 3), dtype='double')
            for i in range(num_atom):
                A_i = np.dot(q, self._born[i])
                for j in range(num_atom):
                    A_j = np.dot(q, self._born[j])
                    nac_q[i, j] = np.outer(A_i, A_j) * constant
            self._set_NAC_force_constants(fc, nac_q)
            self._force_constants = fc
            DynamicalMatrix.set_dynamical_matrix(self, q_red, verbose)

    def _set_NAC_force_constants(self, fc, nac_q):
        N = (self._scell.get_number_of_atoms() //
             self._pcell.get_number_of_atoms())
        for s1 in range(self._scell.get_number_of_atoms()):
            # This if-statement is the trick.
            # In contructing dynamical matrix in phonopy
            # fc of left indices with s1 == self._s2p_map[ s1 ] are
            # only used.
            if s1 != self._s2p_map[s1]:
                continue
            p1 = self._p2p_map[s1]
            for s2 in range(self._scell.get_number_of_atoms()):            
                p2 = self._p2p_map[s2]
                fc[s1, s2] += nac_q[p1, p2] / N

    def _get_charge_sum(self, num_atom, q):
        charge_sum = np.zeros((num_atom, num_atom, 3, 3), dtype='double')
        for i in range(num_atom):
            for j in range(num_atom):
                for a in (0, 1, 2):
                    for b in (0, 1, 2):
                        charge_sum[i, j, a, b] = \
                            np.dot(q, self._born[i, :, a]) * np.dot(q, self._born[j, :, b])
        return charge_sum

    def _set_c_nac_dynamical_matrix(self, q_red, q, factor):
        import phonopy._phonopy as phonoc

        fc = self._bare_force_constants.copy()
        vectors = self._smallest_vectors
        mass = self._pcell.get_masses()
        multiplicity = self._multiplicity
        size_prim = len(mass)
        itemsize = self._force_constants.itemsize
        dm = np.zeros((size_prim * 3, size_prim * 3),
                      dtype=("c%d" % (itemsize * 2)))
        phonoc.nac_dynamical_matrix(dm.view(dtype='double'),
                                    fc,
                                    np.array(q_red, dtype='double'),
                                    vectors,
                                    multiplicity,
                                    mass,
                                    self._s2p_map,
                                    self._p2s_map,
                                    np.array(q, dtype='double'),
                                    self._born,
                                    factor)
        self._dynamical_matrix = dm


# Helper methods
def get_equivalent_smallest_vectors(atom_number_supercell,
                                    atom_number_primitive,
                                    supercell,
                                    primitive_lattice,
                                    symprec):
    distances = []
    differences = []
    reduced_bases = get_reduced_bases(supercell.get_cell(), symprec)
    positions = np.dot(supercell.get_positions(), np.linalg.inv(reduced_bases))

    # Atomic positions are confined into the lattice made of reduced bases.
    for pos in positions:
        pos -= np.rint(pos)

    p_pos = positions[atom_number_primitive]
    s_pos = positions[atom_number_supercell]
    for i in (-1, 0, 1):
        for j in (-1, 0, 1):
            for k in (-1, 0, 1):
                # The vector arrow is from the atom in primitive to
                # the atom in supercell cell plus a supercell lattice
                # point. This is related to determine the phase
                # convension when building dynamical matrix.
                diff = s_pos + [i, j, k] - p_pos
                differences.append(diff)
                vec = np.dot(diff, reduced_bases)
                distances.append(np.linalg.norm(vec))

    minimum = min(distances)
    smallest_vectors = []
    for i in range(27):
        if abs(minimum - distances[i]) < symprec:
            relative_scale = np.dot(reduced_bases,
                                    np.linalg.inv(primitive_lattice))
            smallest_vectors.append(np.dot(differences[i], relative_scale))
            
    return smallest_vectors

def get_smallest_vectors(supercell, primitive, symprec):
    """
    shortest_vectors:

      Shortest vectors from an atom in primitive cell to an atom in
      supercell in the fractional coordinates. If an atom in supercell
      is on the border centered at an atom in primitive and there are
      multiple vectors that have the same distance and different
      directions, several shortest vectors are stored. The
      multiplicity is stored in another array, "multiplicity".
      [atom_super, atom_primitive, multiple-vectors, 3]
      
    multiplicity:
      Number of multiple shortest vectors (third index of "shortest_vectors")
      [atom_super, atom_primitive]
    """

    p2s_map = primitive.get_primitive_to_supercell_map()
    size_super = supercell.get_number_of_atoms()
    size_prim = primitive.get_number_of_atoms()
    shortest_vectors = np.zeros((size_super, size_prim, 27, 3), dtype='double')
    multiplicity = np.zeros((size_super, size_prim), dtype='intc')

    for i in range(size_super): # run in supercell
        for j, s_j in enumerate(p2s_map): # run in primitive
            vectors = get_equivalent_smallest_vectors(i,
                                                      s_j,
                                                      supercell, 
                                                      primitive.get_cell(),
                                                      symprec)
            multiplicity[i][j] = len(vectors)
            for k, elem in enumerate(vectors):
                shortest_vectors[i][j][k] = elem

    return shortest_vectors, multiplicity





# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import numpy as np

try:
    import yaml
except ImportError:
    print("You need to install python-yaml.")
    sys.exit(1)
    
try:
    from yaml import CLoader as Loader
    from yaml import CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from phonopy.units import VaspToTHz

def read_band_yaml(filename):
    data = yaml.load(open(filename), Loader=Loader)
    frequencies = []
    distances = []
    labels = []
    for j, v in enumerate(data['phonon']):
        if 'label' in v:
            labels.append(v['label'])
        else:
            labels.append(None)
        frequencies.append([f['frequency'] for f in v['band']])
        distances.append(v['distance'])

    return (np.array(distances),
            np.array(frequencies),
            data['segment_nqpoint'],
            labels)

def read_dos_dat(filename):
    dos = []
    frequencies = []
    for line in open(filename):
        if line.strip()[0] == '#':
            continue
        ary = [float(x) for x in line.split()]
        frequencies.append(ary.pop(0))
        dos.append(ary)
    return np.array(frequencies), np.array(dos)

# Parse options
from optparse import OptionParser
parser = OptionParser()
parser.set_defaults(band_labels=None,
                    dos=None,
                    dos_max=None, 
                    dos_min=None,
                    factor=1.0,
                    f_max=None, 
                    f_min=None,
                    is_gnuplot=False,
                    is_points=False,
                    is_vertial_line=False,
                    output_filename=None,
                    xlabel=None,
                    ylabel=None,
                    show_legend=False,
                    title=None)
parser.add_option("--band_labels", dest="band_labels", action="store", type="string",
                  help="Show labels at band segments")
parser.add_option("--dos", dest="dos", type="string",
                  help="Read dos.dat type file and plot with band structure")
parser.add_option("--dmax", dest="dos_max", type="float",
                  help="Maximum DOS plotted")
parser.add_option("--dmin", dest="dos_min", type="float",
                  help="Minimum DOS plotted")
parser.add_option("--factor", dest="factor", type="float",
                  help="Conversion factor to favorite frequency unit")
parser.add_option("--fmax", dest="f_max", type="float",
                  help="Maximum frequency plotted")
parser.add_option("--fmin", dest="f_min", type="float",
                  help="Minimum frequency plotted")
parser.add_option("--gnuplot", dest="is_gnuplot", action="store_true",
                  help="Output in gnuplot data style")
parser.add_option("--legend", dest="show_legend",
                  action="store_true",
                  help="Show legend")
parser.add_option("--line", "-l", dest="is_vertial_line",
                  action="store_true",
                  help="Vertial line is drawn at between paths")
parser.add_option("-o", "--output", dest="output_filename",
                  action="store", type="string",
                  help="Output filename of PDF plot")
parser.add_option("--xlabel", dest="xlabel", action="store", type="string",
                  help="Specify x-label")
parser.add_option("--ylabel", dest="ylabel", action="store", type="string",
                  help="Specify y-label")
parser.add_option("--points", dest="is_points",
                  action="store_true",
                  help="Draw points")
parser.add_option("-t", "--title", dest="title", action="store",
                  type="string", help="Title of plot")
(options, args) = parser.parse_args()

if options.output_filename:
    import matplotlib
    matplotlib.use('Agg')            

if not options.is_gnuplot:
    import matplotlib.pyplot as plt
    if options.band_labels:
        from matplotlib import rc
        rc('text', usetex=True)
    if options.dos:
        import matplotlib.gridspec as gridspec
        plt.figure(figsize=(10, 6))
        gs = gridspec.GridSpec(1, 2, width_ratios=[3, 1])
        ax1 = plt.subplot(gs[0, 0])

colors = ['r-', 'b-', 'g-', 'c-', 'm-', 'y-', 'k-', 
          'r--', 'b--', 'g--', 'c--', 'm--', 'y--', 'k--']
if options.is_points:
    colors = [x + 'o' for x in colors]

count = 0


if len(args) == 0:
    filenames = ['band.yaml']
else:
    filenames = args

if options.is_gnuplot:
    print("# distance  frequency (bands are separated by blank lines)")

if options.dos:
    dos_frequencies, dos = read_dos_dat(options.dos)

for i, filename in enumerate(filenames):
    (distances,
     frequencies,
     segment_nqpoint,
     labels) = read_band_yaml(filename)

    end_points = [0,]
    for nq in segment_nqpoint:
        end_points.append(nq + end_points[-1])
    end_points[-1] -= 1
    segment_positions = distances[end_points]

    if all(x is None for x in labels):
        labels_at_ends = None
    else:
        labels_at_ends = [labels[n] for n in end_points]

    if options.is_gnuplot:
        print("# End points of segments: ")
        print("#   " + "%10.8f " * len(segment_positions) %
              tuple(segment_positions))
    elif options.is_vertial_line and len(filenames) == 1:
        for v in segment_positions[1:-1]:
            plt.axvline(x=v, linewidth=0.5, color='b')

    if options.is_gnuplot:
        for j, freqs in enumerate(frequencies.T):
            q = 0
            for nq in segment_nqpoint:
                for d, f in zip(distances[q:(q + nq)],
                                freqs[q:(q + nq)] * options.factor):
                    print("%f %f" % (d, f))
                q += nq
                print('')
            print('')
    else:
        q = 0
        for j, nq in enumerate(segment_nqpoint):
            if j == 0:
                plt.plot(distances[q:(q + nq)],
                         frequencies[q:(q + nq)] * options.factor,
                         colors[i],
                         label=filename)
            else:
                plt.plot(distances[q:(q + nq)],
                         frequencies[q:(q + nq)] * options.factor,
                         colors[i])
            q += nq

    if options.is_gnuplot:
        print('')

    
if not options.is_gnuplot:
    if options.xlabel is None:
        plt.xlabel('Wave vector')
    else:
        plt.xlabel(options.xlabel)
    if options.ylabel is None:
        plt.ylabel('Frequency')
    else:
        plt.ylabel(options.ylabel)
    
    plt.xlim(distances[0], distances[-1])
    if options.f_max is not None:
        plt.ylim(ymax=options.f_max)
    if options.f_min is not None:
        plt.ylim(ymin=options.f_min)
    plt.axhline(y=0, linestyle=':', linewidth=0.5, color='b')
    if len(filenames) == 1:
        xticks = segment_positions
        if options.band_labels:
            band_labels = [x for x in options.band_labels.split()]
            if len(band_labels) == len(xticks):
                plt.xticks(xticks, band_labels)
            else:
                print("Numbers of labels and band segments don't match.")
                sys.exit(1)
        elif labels_at_ends:
            plt.xticks(xticks, labels_at_ends)
        else:
            plt.xticks(xticks, [''] * len(xticks))
    else:
        plt.xticks([])
    
    if options.title is not None:
        plt.title(options.title)

    if options.show_legend:
        plt.legend()

    if options.dos:
        ax2 = plt.subplot(gs[0, 1], sharey=ax1)
        plt.subplots_adjust(wspace=0.03)
        plt.setp(ax2.get_yticklabels(), visible=False)
        plt.plot(dos, dos_frequencies)
        plt.xlabel('DOS')
        if options.f_max is not None:
            plt.ylim(ymax=options.f_max)
        if options.f_min is not None:
            plt.ylim(ymin=options.f_min)
        if options.dos_max is not None:
            plt.xlim(xmax=options.dos_max)
        if options.dos_min is not None:
            plt.xlim(xmin=options.dos_min)
    
    if options.output_filename is not None:
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.savefig(options.output_filename)
    else:
        plt.show()



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import numpy as np
from phonopy.phonon.tetrahedron_mesh import TetrahedronMesh


def write_total_dos(frequency_points,
                    total_dos,
                    comment=None):
    fp = open('total_dos.dat', 'w')
    if comment is not None:
        fp.write("# %s\n" % comment)

    for freq, dos in zip(frequency_points, total_dos):
        fp.write("%20.10f%20.10f\n" % (freq, dos))

def write_partial_dos(frequency_points,
                      partial_dos,
                      comment=None):
    fp = open('partial_dos.dat', 'w')
    if comment is not None:
        fp.write("# %s\n" % comment)

    for freq, pdos in zip(frequency_points, partial_dos.T):
        fp.write("%20.10f" % freq)
        fp.write(("%20.10f" * len(pdos)) % tuple(pdos))
        fp.write("\n")


def plot_total_dos(pyplot,
                   frequency_points,
                   total_dos,
                   freq_Debye=None,
                   Debye_fit_coef=None,
                   xlabel=None,
                   ylabel=None,
                   draw_grid=True,
                   flip_xy=False):
    if freq_Debye is not None:
        freq_pitch = frequency_points[1] - frequency_points[0]
        num_points = int(freq_Debye / freq_pitch)
        freqs = np.linspace(0, freq_Debye, num_points + 1)

    if flip_xy:
        pyplot.plot(total_dos, frequency_points, 'r-')
        if freq_Debye:
            pyplot.plot(np.append(Debye_fit_coef * freqs**2, 0),
                        np.append(freqs, freq_Debye), 'b-')
    else:
        pyplot.plot(frequency_points, total_dos, 'r-')
        if freq_Debye:
            pyplot.plot(np.append(freqs, freq_Debye),
                        np.append(Debye_fit_coef * freqs**2, 0), 'b-')

    if xlabel:
        pyplot.xlabel(xlabel)
    if ylabel:
        pyplot.ylabel(ylabel)

    pyplot.grid(draw_grid)

def plot_partial_dos(pyplot,
                     frequency_points,
                     partial_dos,
                     indices=None,
                     legend=None,
                     xlabel=None,
                     ylabel=None,
                     draw_grid=True,
                     flip_xy=False):
    plots = []
    num_pdos = len(partial_dos)

    if indices is None:
        indices = []
        for i in range(num_pdos):
            indices.append([i])

    for set_for_sum in indices:
        pdos_sum = np.zeros_like(frequency_points)
        for i in set_for_sum:
            if i > num_pdos - 1:
                print("Index number \'%d\' is specified," % (i + 1))
                print("but it is not allowed to be larger than the number of "
                      "atoms.")
                raise ValueError
            if i < 0:
                print("Index number \'%d\' is specified, but it must be "
                      "positive." % (i + 1))
                raise ValueError
            pdos_sum += partial_dos[i]
        if flip_xy:
            plots.append(pyplot.plot(pdos_sum, frequency_points))
        else:
            plots.append(pyplot.plot(frequency_points, pdos_sum))

    if legend is not None:
        pyplot.legend(legend)

    if xlabel:
        pyplot.xlabel(xlabel)
    if ylabel:
        pyplot.ylabel(ylabel)

    pyplot.grid(draw_grid)

class NormalDistribution(object):
    def __init__(self, sigma):
        self._sigma = sigma

    def calc(self, x):
        return 1.0 / np.sqrt(2 * np.pi) / self._sigma * \
            np.exp(-x**2 / 2.0 / self._sigma**2)

class CauchyDistribution(object):
    def __init__(self, gamma):
        self._gamma = gamma

    def calc(self, x):
        return self._gamma / np.pi / (x**2 + self._gamma**2)


class Dos(object):
    def __init__(self, mesh_object, sigma=None, tetrahedron_method=False):
        self._mesh_object = mesh_object
        self._frequencies = mesh_object.get_frequencies()
        self._weights = mesh_object.get_weights()
        if tetrahedron_method:
            self._tetrahedron_mesh = TetrahedronMesh(
                mesh_object.get_dynamical_matrix().get_primitive(),
                self._frequencies,
                mesh_object.get_mesh_numbers(),
                mesh_object.get_grid_address(),
                mesh_object.get_grid_mapping_table(),
                mesh_object.get_ir_grid_points())
        else:
            self._tetrahedron_mesh = None

        self._frequency_points = None
        self._sigma = sigma
        self.set_draw_area()
        self.set_smearing_function('Normal')

    def set_smearing_function(self, function_name):
        """
        function_name ==
        'Normal': smearing is done by normal distribution.
        'Cauchy': smearing is done by Cauchy distribution.
        """
        if function_name == 'Cauchy':
            self._smearing_function = CauchyDistribution(self._sigma)
        else:
            self._smearing_function = NormalDistribution(self._sigma)

    def set_sigma(self, sigma):
        self._sigma = sigma

    def set_draw_area(self,
                      freq_min=None,
                      freq_max=None,
                      freq_pitch=None):

        f_min = self._frequencies.min()
        f_max = self._frequencies.max()

        if self._sigma is None:
            self._sigma = (f_max - f_min) / 100.0

        if freq_min is None:
            f_min -= self._sigma * 10
        else:
            f_min = freq_min

        if freq_max is None:
            f_max += self._sigma * 10
        else:
            f_max = freq_max

        if freq_pitch is None:
            f_delta = (f_max - f_min) / 200.0
        else:
            f_delta = freq_pitch
        self._frequency_points = np.arange(f_min,
                                           f_max + f_delta * 0.1,
                                           f_delta)

class TotalDos(Dos):
    def __init__(self, mesh_object, sigma=None, tetrahedron_method=False):
        Dos.__init__(self,
                     mesh_object,
                     sigma=sigma,
                     tetrahedron_method=tetrahedron_method)
        self._dos = None
        self._freq_Debye = None
        self._Debye_fit_coef = None

    def run(self):
        if self._tetrahedron_mesh is None:
            self._dos = np.array([self._get_density_of_states_at_freq(f)
                                  for f in self._frequency_points])
        else:
            self._dos = np.zeros_like(self._frequency_points)
            thm = self._tetrahedron_mesh
            thm.set(value='I', frequency_points=self._frequency_points)
            for i, iw in enumerate(thm):
                self._dos += np.sum(iw * self._weights[i], axis=1)

    def get_dos(self):
        """
        Return freqs and total dos
        """
        return self._frequency_points, self._dos

    def get_Debye_frequency(self):
        return self._freq_Debye

    def set_Debye_frequency(self, num_atoms, freq_max_fit=None):
        try:
            from scipy.optimize import curve_fit
        except ImportError:
            print("You need to install python-scipy.")
            sys.exit(1)

        def Debye_dos(freq, a):
            return a * freq**2

        freq_min = self._frequency_points.min()
        freq_max = self._frequency_points.max()

        if freq_max_fit is None:
            N_fit = int(len(self._frequency_points) / 4.0) # Hard coded
        else:
            N_fit = int(freq_max_fit / (freq_max - freq_min) *
                        len(self._frequency_points))
        popt, pcov = curve_fit(Debye_dos,
                               self._frequency_points[0:N_fit],
                               self._dos[0:N_fit])
        a2 = popt[0]
        self._freq_Debye = (3 * 3 * num_atoms / a2)**(1.0 / 3)
        self._Debye_fit_coef = a2

    def plot(self,
             pyplot,
             xlabel=None,
             ylabel=None,
             draw_grid=True,
             flip_xy=False):
        if flip_xy:
            _xlabel = 'Density of states'
            _ylabel = 'Frequency'
        else:
            _xlabel = 'Frequency'
            _ylabel = 'Density of states'

        if xlabel is not None:
            _xlabel = xlabel
        if ylabel is not None:
            _ylabel = ylabel

        plot_total_dos(pyplot,
                       self._frequency_points,
                       self._dos,
                       freq_Debye=self._freq_Debye,
                       Debye_fit_coef=self._Debye_fit_coef,
                       xlabel=_xlabel,
                       ylabel=_ylabel,
                       draw_grid=draw_grid,
                       flip_xy=flip_xy)

    def write(self):
        if self._tetrahedron_mesh is None:
            comment = "Sigma = %f" % self._sigma
        else:
            comment = "Tetrahedron method"

        write_total_dos(self._frequency_points,
                        self._dos,
                        comment=comment)

    def _get_density_of_states_at_freq(self, f):
        return np.sum(np.dot(
            self._weights, self._smearing_function.calc(self._frequencies - f))
            ) /  np.sum(self._weights)


class PartialDos(Dos):
    def __init__(self,
                 mesh_object,
                 sigma=None,
                 tetrahedron_method=False,
                 direction=None,
                 xyz_projection=False):
        Dos.__init__(self,
                     mesh_object,
                     sigma=sigma,
                     tetrahedron_method=tetrahedron_method)
        self._eigenvectors = self._mesh_object.get_eigenvectors()
        self._partial_dos = None

        if xyz_projection:
            self._eigvecs2 = np.abs(self._eigenvectors) ** 2
        else:
            num_atom = self._frequencies.shape[1] // 3
            i_x = np.arange(num_atom, dtype='int') * 3
            i_y = np.arange(num_atom, dtype='int') * 3 + 1
            i_z = np.arange(num_atom, dtype='int') * 3 + 2
            if direction is None:
                self._eigvecs2 = np.abs(self._eigenvectors[:, i_x, :]) ** 2
                self._eigvecs2 += np.abs(self._eigenvectors[:, i_y, :]) ** 2
                self._eigvecs2 += np.abs(self._eigenvectors[:, i_z, :]) ** 2
            else:
                d = np.array(direction, dtype='double')
                d /= np.linalg.norm(direction)
                proj_eigvecs = self._eigenvectors[:, i_x, :] * d[0]
                proj_eigvecs += self._eigenvectors[:, i_y, :] * d[1]
                proj_eigvecs += self._eigenvectors[:, i_z, :] * d[2]
                self._eigvecs2 = np.abs(proj_eigvecs) ** 2

    def run(self):
        num_pdos = self._eigvecs2.shape[1]
        num_freqs = len(self._frequency_points)
        self._partial_dos = np.zeros((num_pdos, num_freqs), dtype='double')
        if self._tetrahedron_mesh is None:
            self._run_smearing_method()
        else:
            self._run_tetrahedron_method()

    def _run_smearing_method(self):
        weights = self._weights / float(np.sum(self._weights))
        for i, freq in enumerate(self._frequency_points):
            amplitudes = self._smearing_function.calc(self._frequencies - freq)
            for j in range(self._partial_dos.shape[0]):
                self._partial_dos[j, i]= np.dot(
                    weights, self._eigvecs2[:, j, :] * amplitudes).sum()

    def _run_tetrahedron_method(self):
        thm = self._tetrahedron_mesh
        thm.set(value='I', frequency_points=self._frequency_points)
        for i, iw in enumerate(thm):
            w = self._weights[i]
            # for ib, frac in enumerate(self._eigvecs2[i].T):
            #     for j in range(len(self._frequency_points)):
            #         self._partial_dos[:, j] += iw[j, ib] * frac * w
            self._partial_dos += np.dot(iw * w, self._eigvecs2[i].T).T

    def get_partial_dos(self):
        """
        frequency_points: Sampling frequencies
        partial_dos: [atom_index, frequency_points_index]
        """
        return self._frequency_points, self._partial_dos

    def plot(self,
             pyplot,
             indices=None,
             legend=None,
             xlabel=None,
             ylabel=None,
             draw_grid=True,
             flip_xy=False):

        if flip_xy:
            _xlabel = 'Partial density of states'
            _ylabel = 'Frequency'
        else:
            _xlabel = 'Frequency'
            _ylabel = 'Partial density of states'

        if xlabel is not None:
            _xlabel = xlabel
        if ylabel is not None:
            _ylabel = ylabel

        plot_partial_dos(pyplot,
                         self._frequency_points,
                         self._partial_dos,
                         indices=indices,
                         legend=legend,
                         xlabel=_xlabel,
                         ylabel=_ylabel,
                         draw_grid=draw_grid,
                         flip_xy=flip_xy)

    def write(self):
        if self._tetrahedron_mesh is None:
            comment = "Sigma = %f" % self._sigma
        else:
            comment = "Tetrahedron method"

        write_partial_dos(self._frequency_points,
                          self._partial_dos,
                          comment=comment)



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import io
import numpy as np
from phonopy.structure.atoms import PhonopyAtoms as Atoms
from phonopy.structure.atoms import symbol_map, atom_data
from phonopy.structure.cells import get_primitive, get_supercell
from phonopy.structure.symmetry import Symmetry
from phonopy.harmonic.force_constants import similarity_transformation
from phonopy.file_IO import (write_FORCE_SETS, write_force_constants_to_hdf5,
                             write_FORCE_CONSTANTS)

def parse_set_of_forces(num_atoms,
                        forces_filenames,
                        use_expat=True,
                        verbose=True):
    if verbose:
        if use_expat:
            sys.stdout.write(
                "*********************************************\n"
                "*** VasprunxmlExpat is under the testing. ***\n"
                "***  Please report if you find problems.  ***\n"
                "*********************************************\n\n")
        sys.stdout.write("counter (file index): ")

    count = 0
    is_parsed = True
    force_sets = []
    force_files = forces_filenames

    for filename in force_files:
        with io.open(filename, "rb") as fp:
            vasprun = Vasprun(fp, use_expat=use_expat)
            force_sets.append(vasprun.read_forces())
            if verbose:
                sys.stdout.write("%d " % (count + 1))
            count += 1
    
            if not check_forces(force_sets[-1], num_atoms, filename):
                is_parsed = False

    if verbose:
        print('')

    if is_parsed:
        return force_sets
    else:
        return []

def check_forces(forces, num_atom, filename, verbose=True):
    if len(forces) != num_atom:
        if verbose:
            stars = '*' * len(filename)
            sys.stdout.write("\n")
            sys.stdout.write("***************%s***************\n" % stars)
            sys.stdout.write("***** Parsing \"%s\" failed. *****\n" % filename)
            sys.stdout.write("***************%s***************\n" % stars)
        return False
    else:
        return True

def get_drift_forces(forces, filename=None, verbose=True):
    drift_force = np.sum(forces, axis=0) / len(forces)

    if verbose:
        if filename is None:
            print("Drift force: %12.8f %12.8f %12.8f to be subtracted"
                  % tuple(drift_force))
        else:
            print("Drift force of \"%s\" to be subtracted" % filename)
            print("%12.8f %12.8f %12.8f" % tuple(drift_force))
        sys.stdout.flush()

    return drift_force

def create_FORCE_CONSTANTS(filename, options, log_level):
    vasprun = Vasprun(io.open(filename, "rb"))
    fc_and_atom_types = vasprun.read_force_constants()
    if not fc_and_atom_types:
        print('')
        print("\'%s\' dones not contain necessary information." % filename)
        return 1

    force_constants, atom_types = fc_and_atom_types
    if options.is_hdf5:
        try:
            import h5py
        except ImportError:
            print('')
            print("You need to install python-h5py.")
            return 1

        write_force_constants_to_hdf5(force_constants)
        if log_level > 0:
            print("force_constants.hdf5 has been created from vasprun.xml.")
    else:
        write_FORCE_CONSTANTS(force_constants)
        if log_level > 0:
            print("FORCE_CONSTANTS has been created from vasprun.xml.")

    if log_level > 0:
        print("Atom types: %s" % (" ".join(atom_types)))
    return 0

#
# read VASP POSCAR
#
def read_vasp(filename, symbols=None):
    with open(filename) as infile :
        lines = infile.readlines()
    return _get_atoms_from_poscar(lines, symbols)

def read_vasp_from_strings(strings, symbols=None):
    return _get_atoms_from_poscar(StringIO(strings).readlines(), symbols)

def _get_atoms_from_poscar(lines, symbols):
    line1 = [x for x in lines[0].split()]
    if _is_exist_symbols(line1):
        symbols = line1

    scale = float(lines[1])

    cell = []
    for i in range(2, 5):
        cell.append([float(x) for x in lines[i].split()[:3]])
    cell = np.array(cell) * scale

    try:
        num_atoms = np.array([int(x) for x in lines[5].split()])
        line_at = 6
    except ValueError:
        symbols = [x for x in lines[5].split()]
        num_atoms = np.array([int(x) for x in lines[6].split()])
        line_at = 7

    expaned_symbols = _expand_symbols(num_atoms, symbols)

    if lines[line_at][0].lower() == 's':
        line_at += 1

    is_scaled = True
    if (lines[line_at][0].lower() == 'c' or
        lines[line_at][0].lower() == 'k'):
        is_scaled = False

    line_at += 1

    positions = []
    for i in range(line_at, line_at + num_atoms.sum()):
        positions.append([float(x) for x in lines[i].split()[:3]])

    if is_scaled:
        atoms = Atoms(symbols=expaned_symbols,
                      cell=cell,
                      scaled_positions=positions)
    else:
        atoms = Atoms(symbols=expaned_symbols,
                      cell=cell,
                      positions=positions)

    return atoms

def _is_exist_symbols(symbols):
    for s in symbols:
        if not (s in symbol_map):
            return False
    return True

def _expand_symbols(num_atoms, symbols=None):
    expanded_symbols = []
    is_symbols = True
    if symbols is None:
        is_symbols = False
    else:
        if len(symbols) != len(num_atoms):
            is_symbols = False
        else:
            for s in symbols:
                if not s in symbol_map:
                    is_symbols = False
                    break

    if is_symbols:
        for s, num in zip(symbols, num_atoms):
            expanded_symbols += [s] * num
    else:
        for i, num in enumerate(num_atoms):
            expanded_symbols += [atom_data[i+1][1]] * num

    return expanded_symbols

#
# write vasp POSCAR
#
def write_vasp(filename, atoms, direct=True):
    lines = _get_vasp_structure(atoms, direct=direct)
    with open(filename, 'w') as w:
        w.write("\n".join(lines))

def write_supercells_with_displacements(supercell,
                                        cells_with_displacements,
                                        pre_filename="POSCAR",
                                        width=3):
    write_vasp("SPOSCAR", supercell, direct=True)
    for i, cell in enumerate(cells_with_displacements):
        if cell is not None:
            write_vasp("{pre_filename}-{0:0{width}}".format(i + 1,
                                                   pre_filename=pre_filename,
                                                   width=width),
                       cell,
                       direct=True)

    _write_magnetic_moments(supercell)

def _write_magnetic_moments(cell):
    magmoms = cell.get_magnetic_moments()
    if magmoms is not  None:
        w = open("MAGMOM", 'w')
        (num_atoms,
         symbols,
         scaled_positions,
         sort_list) = sort_positions_by_symbols(cell.get_chemical_symbols(),
                                                cell.get_scaled_positions())
        w.write(" MAGMOM = ")
        for i in sort_list:
            w.write("%f " % magmoms[i])
        w.write("\n")
        w.close()

def get_scaled_positions_lines(scaled_positions):
    return "\n".join(_get_scaled_positions_lines(scaled_positions))

def _get_scaled_positions_lines(scaled_positions):
    lines = []
    for i, vec in enumerate(scaled_positions):
        line_str = ""
        for x in (vec - np.rint(vec)):
            if float('%20.16f' % x) < 0.0:
                line_str += "%20.16f" % (x + 1.0)
            else:
                line_str += "%20.16f" % (x)
        lines.append(line_str)

    return lines

def sort_positions_by_symbols(symbols, positions):
    reduced_symbols = _get_reduced_symbols(symbols)
    sorted_positions = []
    sort_list = []
    num_atoms = np.zeros(len(reduced_symbols), dtype=int)
    for i, rs in enumerate(reduced_symbols):
        for j, (s, p) in enumerate(zip(symbols, positions)):
            if rs == s:
                sorted_positions.append(p)
                sort_list.append(j)
                num_atoms[i] += 1
    return num_atoms, reduced_symbols, np.array(sorted_positions), sort_list

def _get_vasp_structure(atoms, direct=True):
    (num_atoms,
     symbols,
     scaled_positions,
     sort_list) = sort_positions_by_symbols(atoms.get_chemical_symbols(),
                                            atoms.get_scaled_positions())
    lines = []
    lines.append(" ".join(["%s" % s for s in symbols]))
    lines.append("   1.0")
    for a in atoms.get_cell():
        lines.append("  %21.16f %21.16f %21.16f" % tuple(a))
    lines.append(" ".join(["%4d" % n for n in num_atoms]))
    lines.append("Direct")
    lines += _get_scaled_positions_lines(scaled_positions)

    # VASP compiled on some system, ending by \n is necessary to read POSCAR
    # properly.
    lines.append('')

    return lines

def _get_reduced_symbols(symbols):
    reduced_symbols = []
    for s in symbols:
        if not (s in reduced_symbols):
            reduced_symbols.append(s)
    return reduced_symbols

#
# Non-analytical term
#
def get_born_OUTCAR(poscar_filename="POSCAR",
                    outcar_filename="OUTCAR",
                    primitive_matrix=None,
                    supercell_matrix=None,
                    is_symmetry=True,
                    symmetrize_tensors=False,
                    symprec=1e-5):
    if primitive_matrix is None:
        pmat = np.eye(3)
    else:
        pmat = primitive_matrix
    if supercell_matrix is None:
        smat = np.eye(3, dtype='intc')
    else:
        smat = supercell_matrix
    ucell = read_vasp(poscar_filename)
    outcar = open(outcar_filename)

    borns, epsilon = _read_born_and_epsilon(outcar)
    num_atom = len(borns)
    assert num_atom == ucell.get_number_of_atoms()

    if symmetrize_tensors:
        lattice = ucell.get_cell().T
        positions = ucell.get_scaled_positions()
        u_sym = Symmetry(ucell, is_symmetry=is_symmetry, symprec=symprec)
        point_sym = [similarity_transformation(lattice, r)
                     for r in u_sym.get_pointgroup_operations()]
        epsilon = _symmetrize_tensor(epsilon, point_sym)
        borns = _symmetrize_borns(borns, u_sym, lattice, positions, symprec)

    inv_smat = np.linalg.inv(smat)
    scell = get_supercell(ucell, smat, symprec=symprec)
    pcell = get_primitive(scell, np.dot(inv_smat, pmat), symprec=symprec)
    p2s = np.array(pcell.get_primitive_to_supercell_map(), dtype='intc')
    p_sym = Symmetry(pcell, is_symmetry=is_symmetry, symprec=symprec)
    s_indep_atoms = p2s[p_sym.get_independent_atoms()]
    u2u = scell.get_unitcell_to_unitcell_map()
    u_indep_atoms = [u2u[x] for x in s_indep_atoms]
    reduced_borns = borns[u_indep_atoms].copy()

    return reduced_borns, epsilon, s_indep_atoms

def _read_born_and_epsilon(outcar):
    borns = []
    while True:
        line = outcar.readline()
        if not line:
            break

        if "NIONS" in line:
            num_atom = int(line.split()[11])

        if "MACROSCOPIC STATIC DIELECTRIC TENSOR" in line:
            epsilon = []
            outcar.readline()
            epsilon.append([float(x) for x in outcar.readline().split()])
            epsilon.append([float(x) for x in outcar.readline().split()])
            epsilon.append([float(x) for x in outcar.readline().split()])

        if "BORN" in line:
            outcar.readline()
            line = outcar.readline()
            if "ion" in line:
                for i in range(num_atom):
                    born = []
                    born.append([float(x)
                                 for x in outcar.readline().split()][1:])
                    born.append([float(x)
                                 for x in outcar.readline().split()][1:])
                    born.append([float(x)
                                 for x in outcar.readline().split()][1:])
                    outcar.readline()
                    borns.append(born)

    borns = np.array(borns, dtype='double')
    epsilon = np.array(epsilon, dtype='double')

    return borns, epsilon

def _symmetrize_borns(borns, u_sym, lattice, positions, symprec):
    borns_orig = borns.copy()
    for i, Z in enumerate(borns):
        site_sym = [similarity_transformation(lattice, r)
                    for r in u_sym.get_site_symmetry(i)]
        Z = _symmetrize_tensor(Z, site_sym)

    rotations = u_sym.get_symmetry_operations()['rotations']
    translations = u_sym.get_symmetry_operations()['translations']
    map_atoms = u_sym.get_map_atoms()
    borns_copy = np.zeros_like(borns)
    for i in range(len(borns)):
        count = 0
        for r, t in zip(rotations, translations):
            count += 1
            diff = np.dot(positions, r.T) + t - positions[i]
            diff -= np.rint(diff)
            j = np.nonzero((np.abs(diff) < symprec).all(axis=1))[0][0]
            r_cart = similarity_transformation(lattice, r)
            borns_copy[i] += similarity_transformation(r_cart, borns[j])
        borns_copy[i] /= count

    borns = borns_copy
    sum_born = borns.sum(axis=0) / len(borns)
    borns -= sum_born

    if (np.abs(borns_orig - borns) > 0.1).any():
        sys.stderr.write(
            "Born effective charge symmetrization might go wrong.\n")

    return borns

def _symmetrize_tensor(tensor, symmetry_operations):
    sum_tensor = np.zeros_like(tensor)
    for sym in symmetry_operations:
        sum_tensor += similarity_transformation(sym, tensor)
    return sum_tensor / len(symmetry_operations)

#
# vasprun.xml handling
#
class VasprunWrapper(object):
    """VasprunWrapper class
    This is used to avoid VASP 5.2.8 vasprun.xml defect at PRECFOCK,
    xml parser stops with error.
    """
    def __init__(self, fileptr):
        self._fileptr = fileptr

    def read(self, size=None):
        element = self._fileptr.next()
        if element.find("PRECFOCK") == -1:
            return element
        else:
            return "<i type=\"string\" name=\"PRECFOCK\"></i>"

class Vasprun(object):
    def __init__(self, fileptr, use_expat=False):
        self._fileptr = fileptr
        self._use_expat = use_expat

    def read_forces(self):
        if self._use_expat:
            return self._parse_expat_vasprun_xml()
        else:
            vasprun_etree = self._parse_etree_vasprun_xml(tag='varray')
            return self._get_forces(vasprun_etree)

    def read_force_constants(self):
        vasprun = self._parse_etree_vasprun_xml()
        return self._get_force_constants(vasprun)
    
    def _get_forces(self, vasprun_etree):
        """
        vasprun_etree = etree.iterparse(fileptr, tag='varray')
        """
        forces = []
        for event, element in vasprun_etree:
            if element.attrib['name'] == 'forces':
                for v in element:
                    forces.append([float(x) for x in v.text.split()])
        return np.array(forces)
    
    def _get_force_constants(self, vasprun_etree):
        fc_tmp = None
        num_atom = 0
        for event, element in vasprun_etree:
            if num_atom == 0:
                atomtypes = self._get_atomtypes(element)
                if atomtypes:
                    num_atoms, elements, elem_masses = atomtypes[:3]
                    num_atom = np.sum(num_atoms)
                    masses = []
                    for n, m in zip(num_atoms, elem_masses):
                        masses += [m] * n
    
            # Get Hessian matrix (normalized by masses)
            if element.tag == 'varray':
                if element.attrib['name'] == 'hessian':
                    fc_tmp = []
                    for v in element.findall('./v'):
                        fc_tmp.append([float(x) for x in v.text.strip().split()])
    
        if fc_tmp is None:
            return False
        else:
            fc_tmp = np.array(fc_tmp)
            if fc_tmp.shape != (num_atom * 3, num_atom * 3):
                return False
            # num_atom = fc_tmp.shape[0] / 3
            force_constants = np.zeros((num_atom, num_atom, 3, 3), dtype='double')
    
            for i in range(num_atom):
                for j in range(num_atom):
                    force_constants[i, j] = fc_tmp[i*3:(i+1)*3, j*3:(j+1)*3]
    
            # Inverse normalization by atomic weights
            for i in range(num_atom):
                for j in range(num_atom):
                    force_constants[i, j] *= -np.sqrt(masses[i] * masses[j])
    
            return force_constants, elements
    
    def _get_atomtypes(self, element):
        atom_types = []
        masses = []
        valences = []
        num_atoms = []
    
        if element.tag == 'array':
            if 'name' in element.attrib:
                if element.attrib['name'] == 'atomtypes':
                    for rc in element.findall('./set/rc'):
                        atom_info = [x.text for x in rc.findall('./c')]
                        num_atoms.append(int(atom_info[0]))
                        atom_types.append(atom_info[1].strip())
                        masses.append(float(atom_info[2]))
                        valences.append(float(atom_info[3]))
                    return num_atoms, atom_types, masses, valences
    
        return None

    def _parse_etree_vasprun_xml(self, tag=None):
        if self._is_version528():
            return self._parse_by_etree(VasprunWrapper(self._fileptr), tag=tag)
        else:
            return self._parse_by_etree(self._fileptr, tag=tag)
    
    def _parse_by_etree(self, fileptr, tag=None):
        try:
            import xml.etree.cElementTree as etree
            for event, elem in etree.iterparse(fileptr):
                if tag is None or elem.tag == tag:
                    yield event, elem
        except ImportError:
            print("Python 2.5 or later is needed.")
            print("For creating FORCE_SETS file with Python 2.4, you can use "
                  "phonopy 1.8.5.1 with python-lxml .")
            sys.exit(1)
    
    def _parse_expat_vasprun_xml(self):
        if self._is_version528():
            return self._parse_by_expat(VasprunWrapper(self._fileptr))
        else:
            return self._parse_by_expat(self._fileptr)
    
    def _parse_by_expat(self, fileptr):
        vasprun = VasprunxmlExpat(fileptr)
        vasprun.parse()
        return vasprun.get_forces()[-1]

    def _is_version528(self):
        for line in self._fileptr:
            if '\"version\"' in str(line):
                self._fileptr.seek(0)
                if '5.2.8' in str(line):
                    sys.stdout.write(
                        "\n"
                        "**********************************************\n"
                        "* A special routine was used for VASP 5.2.8. *\n"
                        "**********************************************\n")
                    return True
                else:
                    return False

class VasprunxmlExpat(object):
    def __init__(self, fileptr):
        import xml.parsers.expat

        self._fileptr = fileptr

        self._is_forces = False
        self._is_stress = False
        self._is_positions = False
        self._is_symbols = False
        self._is_basis = False
        self._is_energy = False

        self._is_v = False
        self._is_i = False
        self._is_rc = False
        self._is_c = False

        self._is_scstep = False
        self._is_structure = False

        self._all_forces = []
        self._all_stress = []
        self._all_points = []
        self._all_lattice = []
        self._symbols = []
        self._all_energies = []
        self._forces = None
        self._stress = None
        self._points = None
        self._lattice = None
        self._energies = None

        self._p = xml.parsers.expat.ParserCreate()
        self._p.buffer_text = True
        self._p.StartElementHandler = self._start_element
        self._p.EndElementHandler = self._end_element
        self._p.CharacterDataHandler = self._char_data

    def parse(self):
        try:
            self._p.ParseFile(self._fileptr)
        except:
            return False
        else:
            return True

    def get_forces(self):
        return np.array(self._all_forces)

    def get_stress(self):
        return np.array(self._all_stress)

    def get_points(self):
        return np.array(self._all_points)

    def get_lattice(self):
        return np.array(self._all_lattice)

    def get_symbols(self):
        return self._symbols

    def get_cells(self):
        cells = []
        if len(self._all_points) == len(self._all_lattice):
            for p, l in zip(self._all_points, self._all_lattice):
                cells.append(Cell(lattice=l,
                                  points=p,
                                  symbols=self._symbols))
        return cells

    def get_energies(self):
        return np.array(self._all_energies)

    def _start_element(self, name, attrs):
        # Used not to collect energies in <scstep>
        if name == 'scstep':
            self._is_scstep = True

        # Used not to collect basis and positions in
        # <structure name="initialpos" >
        # <structure name="finalpos" >
        if name == 'structure':
            if 'name' in attrs.keys():
                self._is_structure = True

        if (self._is_forces or
            self._is_stress or
            self._is_positions or
            self._is_basis):
            if name == 'v':
                self._is_v = True

        if name == 'varray':
            if 'name' in attrs.keys():
                if attrs['name'] == 'forces':
                    self._is_forces = True
                    self._forces = []

                if attrs['name'] == 'stress':
                    self._is_stress = True
                    self._stress = []

                if not self._is_structure:
                    if attrs['name'] == 'positions':
                        self._is_positions = True
                        self._points = []

                    if attrs['name'] == 'basis':
                        self._is_basis = True
                        self._lattice = []

        if self._is_energy and name == 'i':
            self._is_i = True

        if name == 'energy' and (not self._is_scstep):
            self._is_energy = True
            self._energies = []

        if self._is_symbols and name == 'rc':
            self._is_rc = True

        if self._is_symbols and self._is_rc and name == 'c':
            self._is_c = True

        if name == 'array':
            if 'name' in attrs.keys():
                if attrs['name'] == 'atoms':
                    self._is_symbols = True


    def _end_element(self, name):
        if name == 'scstep':
            self._is_scstep = False

        if name == 'structure' and self._is_structure:
            self._is_structure = False

        if name == 'varray':
            if self._is_forces:
                self._is_forces = False
                self._all_forces.append(self._forces)

            if self._is_stress:
                self._is_stress = False
                self._all_stress.append(self._stress)

            if self._is_positions:
                self._is_positions = False
                self._all_points.append(np.transpose(self._points))

            if self._is_basis:
                self._is_basis = False
                self._all_lattice.append(np.transpose(self._lattice))

        if name == 'array':
            if self._is_symbols:
                self._is_symbols = False


        if name == 'energy' and (not self._is_scstep):
            self._is_energy = False
            self._all_energies.append(self._energies)

        if name == 'v':
            self._is_v = False

        if name == 'i':
            self._is_i = False

        if name == 'rc':
            self._is_rc = False
            if self._is_symbols:
                self._symbols.pop(-1)

        if name == 'c':
            self._is_c = False

    def _char_data(self, data):
        if self._is_v:
            if self._is_forces:
                self._forces.append(
                    [float(x) for x in data.split()])

            if self._is_stress:
                self._stress.append(
                    [float(x) for x in data.split()])

            if self._is_positions:
                self._points.append(
                    [float(x) for x in data.split()])

            if self._is_basis:
                self._lattice.append(
                    [float(x) for x in data.split()])

        if self._is_i:
            if self._is_energy:
                self._energies.append(float(data.strip()))

        if self._is_c:
            if self._is_symbols:
                self._symbols.append(str(data.strip()))

#
# XDATCAR
#
def read_XDATCAR(filename="XDATCAR"):
    lattice = None
    symbols = None
    numbers_of_atoms = None
    with open(filename) as f:
        f.readline()
        scale = float(f.readline())
        a = [float(x) for x in f.readline().split()[:3]]
        b = [float(x) for x in f.readline().split()[:3]]
        c = [float(x) for x in f.readline().split()[:3]]
        lattice = np.transpose([a, b, c]) * scale
        symbols = f.readline().split()
        numbers_of_atoms = np.array(
            [int(x) for x in f.readline().split()[:len(symbols)]], dtype='intc')

    if lattice is not None:
        data = np.loadtxt(filename, skiprows=7, comments='D')
        return (data.reshape((-1, numbers_of_atoms.sum(), 3)),
                np.array(lattice, dtype='double', order='C'))
    else:
        return None

#
# OUTCAR handling (obsolete)
#
def read_force_constant_OUTCAR(filename):
    return get_force_constants_OUTCAR(filename)

def get_force_constants_OUTCAR(filename):
    file = open(filename)
    while 1:
        line = file.readline()
        if line == '':
            print("Force constants could not be found.")
            return 0

        if line[:19] == " SECOND DERIVATIVES":
            break

    file.readline()
    num_atom = int(((file.readline().split())[-1].strip())[:-1])

    fc_tmp = []
    for i in range(num_atom * 3):
        fc_tmp.append([float(x) for x in (file.readline().split())[1:]])

    fc_tmp = np.array(fc_tmp)

    force_constants = np.zeros((num_atom, num_atom, 3, 3), dtype=float)
    for i in range(num_atom):
        for j in range(num_atom):
            force_constants[i, j] = -fc_tmp[i*3:(i+1)*3, j*3:(j+1)*3]

    return force_constants



# Copyright (C) 2014 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import numpy as np

from phonopy.file_IO import iter_collect_forces
from phonopy.interface.vasp import (get_scaled_positions_lines, check_forces,
                                    get_drift_forces)
from phonopy.units import Bohr
from phonopy.cui.settings import fracval
from phonopy.structure.atoms import PhonopyAtoms as Atoms
from phonopy.structure.atoms import symbol_map

def parse_set_of_forces(num_atoms,
                        forces_filenames,
                        verbose=True):
    hook = 'Forces acting on atoms'
    is_parsed = True
    force_sets = []

    for i, filename in enumerate(forces_filenames):
        if verbose:
            sys.stdout.write("%d. " % (i + 1))
        pwscf_forces = iter_collect_forces(filename,
                                           num_atoms,
                                           hook,
                                           [6, 7, 8],
                                           word='force')
        if check_forces(pwscf_forces, num_atoms, filename, verbose=verbose):
            drift_force = get_drift_forces(pwscf_forces,
                                           filename=filename,
                                           verbose=verbose)
            force_sets.append(np.array(pwscf_forces) - drift_force)
        else:
            is_parsed = False

    if is_parsed:
        return force_sets
    else:
        return []

def read_pwscf(filename):
    pwscf_in = PwscfIn(open(filename).readlines())
    tags = pwscf_in.get_tags()
    lattice = tags['cell_parameters']
    positions = [pos[1] for pos in tags['atomic_positions']]
    species = [pos[0] for pos in tags['atomic_positions']]
    mass_map = {}
    pp_map = {}
    for vals in tags['atomic_species']:
        mass_map[vals[0]] = vals[1]
        pp_map[vals[0]] = vals[2]
    masses = [mass_map[x] for x in species]
    pp_all_filenames = [pp_map[x] for x in species]

    unique_species = []
    for x in species:
        if x not in unique_species:
            unique_species.append(x)

    numbers = []
    is_unusual = False
    for x in species:
        if x in symbol_map:
            numbers.append(symbol_map[x])
        else:
            numbers.append(-unique_species.index(x))
            is_unusual = True

    if is_unusual:
        positive_numbers = []
        for n in numbers:
            if n > 0:
                if n not in positive_numbers:
                    positive_numbers.append(n)

        available_numbers = range(1, 119)
        for pn in positive_numbers:
            available_numbers.remove(pn)

        for i, n in enumerate(numbers):
            if n < 1:
                numbers[i] = available_numbers[-n]

        cell = Atoms(numbers=numbers,
                     masses=masses,
                     cell=lattice,
                     scaled_positions=positions)
    else:
        cell = Atoms(numbers=numbers,
                     cell=lattice,
                     scaled_positions=positions)

    unique_symbols = []
    pp_filenames = {}
    for i, symbol in enumerate(cell.get_chemical_symbols()):
        if symbol not in unique_symbols:
            unique_symbols.append(symbol)
            pp_filenames[symbol] = pp_all_filenames[i]

    return cell, pp_filenames

def write_pwscf(filename, cell, pp_filenames):
    f = open(filename, 'w')
    f.write(get_pwscf_structure(cell, pp_filenames=pp_filenames))

def write_supercells_with_displacements(supercell,
                                        cells_with_displacements,
                                        pp_filenames,
                                        pre_filename="supercell",
                                        width=3):
    write_pwscf("supercell.in", supercell, pp_filenames)
    for i, cell in enumerate(cells_with_displacements):
        if cell is not None:
            filename = "{pre_filename}-{0:0{width}}.in".format(
                i + 1,
                pre_filename=pre_filename,
                width=width)
            write_pwscf(filename,
                        cell,
                        pp_filenames)

def get_pwscf_structure(cell, pp_filenames=None):
    lattice = cell.get_cell()
    positions = cell.get_scaled_positions()
    masses = cell.get_masses()
    chemical_symbols = cell.get_chemical_symbols()
    unique_symbols = []
    atomic_species = []
    for symbol, m in zip(chemical_symbols, masses):
        if symbol not in unique_symbols:
            unique_symbols.append(symbol)
            atomic_species.append((symbol, m))

    lines = ""
    lines += ("!    ibrav = 0, nat = %d, ntyp = %d\n" %
              (len(positions), len(unique_symbols)))
    lines += "CELL_PARAMETERS bohr\n"
    lines += ((" %21.16f" * 3 + "\n") * 3) % tuple(lattice.ravel())
    lines += "ATOMIC_SPECIES\n"
    for symbol, mass in atomic_species:
        if pp_filenames is None:
            lines += " %2s %10.5f   %s_PP_filename\n" % (symbol, mass, symbol)
        else:
            lines += " %2s %10.5f   %s\n" % (symbol, mass, pp_filenames[symbol])
    lines += "ATOMIC_POSITIONS crystal\n"
    for i, (symbol, pos_line) in enumerate(zip(
            chemical_symbols,
            get_scaled_positions_lines(positions).split('\n'))):
        lines += (" %2s " % symbol) + pos_line
        if i < len(chemical_symbols) - 1:
            lines += "\n"

    return lines

class PwscfIn(object):
    def __init__(self, lines):
        self._set_methods = {'ibrav':            self._set_ibrav,
                             'nat':              self._set_nat,
                             'ntyp':             self._set_ntyp,
                             'atomic_species':   self._set_atom_types,
                             'atomic_positions': self._set_positions,
                             'cell_parameters':  self._set_lattice}
        self._tags = {'ibrav':            None,
                      'nat':              None,
                      'ntyp':             None,
                      'atomic_species':   None,
                      'atomic_positions': None,
                      'cell_parameters':  None}

        self._values = None
        self._collect(lines)

    def get_tags(self):
        return self._tags

    def _collect(self, lines):
        elements = {}
        tag = None
        for line_tmp in lines:
            line = line_tmp.split('!')[0]
            if ('atomic_positions' in line.lower() or
                'cell_parameters' in line.lower()):
                if len(line.split()) == 1:
                    words = [line.lower().strip(), 'alat']
                else:
                    words = line.lower().split()[:2]
            elif 'atomic_species' in line.lower():
                words = line.lower().split()
            else:
                line_replaced = line.replace('=', ' ').replace(',', ' ')
                words = line_replaced.split()
            for val in words:
                if val.lower() in self._set_methods:
                    tag = val.lower()
                    elements[tag] = []
                elif tag is not None:
                    elements[tag].append(val)

        for tag in ['ibrav', 'nat', 'ntyp']:
            if tag not in elements:
                print("%s is not found in the input file." % tag)
                sys.exit(1)

        for tag in elements:
            self._values = elements[tag]
            if tag == 'ibrav' or tag == 'nat' or tag == 'ntyp':
                self._set_methods[tag]()

        for tag in elements:
            self._values = elements[tag]
            if tag != 'ibrav' and tag != 'nat' and tag != 'ntyp':
                self._set_methods[tag]()

    def _set_ibrav(self):
        ibrav = int(self._values[0])
        if ibrav != 0:
            print("Only ibrav = 0 is supported.")
            sys.exit(1)

        self._tags['ibrav'] = ibrav

    def _set_nat(self):
        self._tags['nat'] = int(self._values[0])

    def _set_ntyp(self):
        self._tags['ntyp'] = int(self._values[0])

    def _set_lattice(self):
        unit = self._values[0]
        if unit == 'alat':
            print("Only CELL_PARAMETERS format with alat is not supported.")
            sys.exit(1)
        if unit == 'angstrom':
            factor = 1.0 / Bohr
        else:
            factor = 1.0

        if len(self._values[1:]) < 9:
            print("CELL_PARAMETERS is wrongly set.")
            sys.exit(1)

        lattice = np.reshape([float(x) for x in self._values[1:10]], (3, 3))
        self._tags['cell_parameters'] = lattice * factor

    def _set_positions(self):
        unit = self._values[0]
        if unit != 'crystal':
            print("Only ATOMIC_POSITIONS format with "
                  "crystal coordinates is supported.")
            sys.exit(1)

        natom = self._tags['nat']
        pos_vals = self._values[1:]
        if len(pos_vals) < natom * 4:
            print("ATOMIC_POSITIONS is wrongly set.")
            sys.exit(1)

        positions = []
        for i in range(natom):
            positions.append(
                [pos_vals[i * 4],
                 [float(x) for x in pos_vals[i * 4 + 1:i * 4 + 4]]])

        self._tags['atomic_positions'] = positions

    def _set_atom_types(self):
        num_types = self._tags['ntyp']
        if len(self._values) < num_types * 3:
            print("ATOMIC_SPECIES is wrongly set.")
            sys.exit(1)

        species = []

        for i in range(num_types):
            species.append(
                [self._values[i * 3],
                 float(self._values[i * 3 + 1]),
                 self._values[i * 3 + 2]])

        self._tags['atomic_species'] = species



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from phonopy.units import AMU, THzToEv, Kb, EV, Hbar, Angstrom
from phonopy.harmonic.dynamical_matrix import get_equivalent_smallest_vectors
from phonopy.interface.cif import write_cif_P1
# np.seterr(invalid='raise')

class ThermalMotion(object):
    def __init__(self,
                 frequencies, # have to be supplied in THz
                 eigenvectors,
                 masses,
                 cutoff_frequency=None):

        if cutoff_frequency is None:
            self._cutoff_frequency = 0
        else:
            self._cutoff_frequency = cutoff_frequency

        self._distances = None
        self._displacements = None
        self._frequencies = frequencies
        self._p_eigenvectors = None
        self._eigenvectors = eigenvectors
        self._masses = masses * AMU
        self._masses3 = np.array([[m] * 3 for m in masses]).flatten() * AMU
        self._temperatures = None

    def get_Q2(self, freq, t): # freq in THz
        return Hbar * EV / Angstrom ** 2 * (
            (self._get_population(freq, t) + 0.5) / (freq * 1e12 * 2 * np.pi))

    def get_temperatures(self):
        return self._temperatures

    def set_temperature_range(self, t_min=None, t_max=None, t_step=None):
        if t_min is None:
            _t_min = 10
        elif t_min < 0:
            _t_min = 0
        else:
            _t_min = t_min

        if t_max is None:
            _t_max = 1000
        elif t_max > _t_min:
            _t_max = t_max
        else:
            _t_max = _t_min

        if t_step is None:
            _t_step = 10
        elif t_step > 0:
            _t_step = t_step
        else:
            _t_step = 10

        self._temperatures = np.arange(_t_min, _t_max + _t_step / 2.0, _t_step,
                                       dtype='double')

    def set_temperatures(self, temperatures):
        t_array = np.array(temperatures)
        condition = np.logical_not(t_array < 0)
        self._temperatures = np.extract(condition, t_array)

    def project_eigenvectors(self, direction, lattice=None):
        """
        direction

        Without specifying lattice:
          Projection direction in Cartesian coordinates
        With lattice:
          Projection direction in fractional coordinates
        """

        if lattice is not None:
            projector = np.dot(direction, lattice)
        else:
            projector = np.array(direction, dtype=float)
        projector /= np.linalg.norm(projector)

        self._p_eigenvectors = []
        for vecs_q in self._eigenvectors:
            p_vecs_q = []
            for vecs in vecs_q.T:
                p_vecs_q.append(np.dot(vecs.reshape(-1, 3), projector))
            self._p_eigenvectors.append(np.transpose(p_vecs_q))
        self._p_eigenvectors = np.array(self._p_eigenvectors)

    def _get_population(self, freq, t): # freq in THz
        if t < 1: # temperatue less than 1 K is approximated as 0 K.
            return 0
        else:
            return 1.0 / (np.exp(freq * THzToEv / (Kb * t)) - 1)

class ThermalDisplacements(ThermalMotion):
    def __init__(self,
                 frequencies, # Have to be supplied in THz
                 eigenvectors,
                 masses,
                 cutoff_frequency=None):

        ThermalMotion.__init__(self,
                               frequencies,
                               eigenvectors,
                               masses,
                               cutoff_frequency=cutoff_frequency)

        self._displacements = None

    def get_thermal_displacements(self):
        return (self._temperatures, self._displacements)

    def run(self):
        freqs = self._frequencies
        temps = self._temperatures
        if self._p_eigenvectors is not None:
            masses = self._masses
            eigvecs = self._p_eigenvectors
        else:
            masses = self._masses3
            eigvecs = self._eigenvectors

        disps = np.zeros((len(temps), len(masses)), dtype=float)
        for fs, vecs2 in zip(freqs, abs(eigvecs) ** 2):
            for f, v2 in zip(fs, vecs2.T):
                if f > self._cutoff_frequency:
                    c = v2 / masses
                    for i, t in enumerate(temps):
                        disps[i] += self.get_Q2(f, t) * c

        self._displacements = disps / len(freqs)

    def write_yaml(self):
        natom = len(self._masses)
        f = open('thermal_displacements.yaml', 'w')
        f.write("# Thermal displacements\n")
        f.write("natom: %5d\n" % (natom))
        f.write("cutoff_frequency: %f\n" % self._cutoff_frequency)

        f.write("thermal_displacements:\n")
        for t, u in zip(self._temperatures, self._displacements):
            f.write("- temperature:   %15.7f\n" % t)
            f.write("  displacements:\n")
            for i, elems in enumerate(np.reshape(u, (natom, -1))):
                f.write("  - [ %10.7f" % elems[0])
                for j in range(len(elems) - 1):
                    f.write(", %10.7f" % elems[j + 1])
                f.write(" ] # atom %d\n" % (i + 1))

    def plot(self, pyplot, is_legend=False):
        plots = []
        labels = []
        xyz = ['x', 'y', 'z']
        for i, u in enumerate(self._displacements.transpose()):
            plots.append(pyplot.plot(self._temperatures, u ))
            labels.append("%d-%s" % ( i//3 + 1, xyz[i % 3]))

        if is_legend:
            pyplot.legend(plots, labels, loc='upper left')

class ThermalDisplacementMatrices(ThermalMotion):
    def __init__(self,
                 frequencies, # Have to be supplied in THz
                 eigenvectors,
                 masses,
                 cutoff_frequency=None,
                 lattice=None): # column vectors in real space

        ThermalMotion.__init__(self,
                               frequencies,
                               eigenvectors,
                               masses,
                               cutoff_frequency=cutoff_frequency)

        self._disp_matrices = None
        self._disp_matrices_cif = None

        if lattice is not None:
            A = lattice
            N = np.diag([np.linalg.norm(x) for x in np.linalg.inv(A)])
            self._ANinv = np.linalg.inv(np.dot(A, N))
        else:
            self._ANinv = None

    def get_thermal_displacement_matrices(self):
        return (self._temperatures, self._disp_matrices)

    def run(self):
        disps = np.zeros((len(self._temperatures), len(self._masses),
                          3, 3), dtype=complex)

        for freqs, eigvecs in zip(self._frequencies, self._eigenvectors):
            for f, vec in zip(freqs, eigvecs.T):
                if f > self._cutoff_frequency:
                    c = []
                    for v, m in zip(vec.reshape(-1, 3), self._masses):
                        c.append(np.outer(v, v.conj()) / m)
                    for i, t in enumerate(self._temperatures):
                        disps[i] += self.get_Q2(f, t) * np.array(c)
        self._disp_matrices = disps / len(self._frequencies)

        if self._ANinv is not None:
            self._disp_matrices_cif = np.zeros(self._disp_matrices.shape,
                                               dtype='double')
            for i, matrices in enumerate(self._disp_matrices):
                for j, mat in enumerate(matrices):
                    mat_cif = np.dot(np.dot(self._ANinv, mat.real),
                                     self._ANinv.T)
                    self._disp_matrices_cif[i, j] = mat_cif

    def write_cif(self, cell, temperature_index):
        write_cif_P1(cell,
                     U_cif=self._disp_matrices_cif[temperature_index],
                     filename="tdispmat.cif")

    def write_yaml(self):
        natom = len(self._masses)
        lines = []

        with open('thermal_displacement_matrices.yaml', 'w') as w:
            lines.append("# Thermal displacement_matrices")
            lines.append("natom: %5d" % (natom))
            lines.append("cutoff_frequency: %f" % self._cutoff_frequency)
            lines.append("thermal_displacement_matrices:")
            for i, t in enumerate(self._temperatures):
                matrices = self._disp_matrices[i]
                lines.append("- temperature:   %15.7f" % t)
                lines.append("  displacement_matrices:")
                for j, mat in enumerate(matrices):
                    ## For checking imaginary part that should be zero
                    # lines.append("  - # atom %d" % (i + 1))
                    # for v in mat:
                    #     lines.append(
                    #         "    [ %8.5f, %8.5f, %8.5f, %8.5f, %8.5f, %8.5f ]" %
                    #         (tuple(v.real) + tuple(v.imag)))
                    m = mat.real
                    lines.append(
                        ("  - [ " + "%8.5f, " * 5 + "%8.5f ] # atom %d") %
                        (m[0, 0], m[1, 1], m[2, 2],
                         m[1, 2], m[0, 2], m[0, 1], j + 1))
                if self._ANinv is not None:
                    matrices_cif = self._disp_matrices_cif[i]
                    lines.append("  displacement_matrices_cif:")
                    for j, mat_cif in enumerate(matrices_cif):
                        m = mat_cif
                        lines.append(
                            ("  - [ " + "%8.5f, " * 5 + "%8.5f ] # atom %d") %
                            (m[0, 0], m[1, 1], m[2, 2],
                             m[1, 2], m[0, 2], m[0, 1], j + 1))
            w.write("\n".join(lines))

class ThermalDistances(ThermalMotion):
    def __init__(self,
                 frequencies, # Have to be supplied in THz
                 eigenvectors,
                 supercell,
                 primitive,
                 qpoints,
                 symprec=1e-5,
                 cutoff_frequency=None):

        self._primitive = primitive
        self._supercell = supercell
        self._qpoints = qpoints
        self._symprec = symprec

        ThermalMotion.__init__(self,
                               frequencies,
                               eigenvectors,
                               primitive.get_masses(),
                               cutoff_frequency=cutoff_frequency)

    def _get_cross(self, v, delta_r, q, atom1, atom2):
        phase = np.exp(2j * np.pi * np.dot(delta_r, q))
        cross_val = v[atom1]*phase*v[atom2].conjugate()
        return -2*(cross_val).real

    def run(self, atom_pairs):
        s2p = self._primitive.get_supercell_to_primitive_map()
        p2p = self._primitive.get_primitive_to_primitive_map()
        dists = np.zeros((len(self._temperatures), len(atom_pairs)), dtype=float)
        for i, (atom1, atom2) in enumerate(atom_pairs):
            patom1 = p2p[s2p[atom1]]
            patom2 = p2p[s2p[atom2]]
            delta_r = get_equivalent_smallest_vectors(atom2,
                                                      atom1,
                                                      self._supercell,
                                                      self._primitive.get_cell(),
                                                      self._symprec)[0]

            self._project_eigenvectors(delta_r, self._primitive.get_cell())

            for freqs, vecs, q in zip(self._frequencies,
                                      self._p_eigenvectors,
                                      self._qpoints):
                c_cross = 1.0 / np.sqrt(self._masses[patom1] *
                                        self._masses[patom2])
                c1 = 1.0 / self._masses[patom1]
                c2 = 1.0 / self._masses[patom2]

                for f, v in zip(freqs, vecs.T):
                    cross_term = self._get_cross(v, delta_r, q, patom1, patom2)
                    v2 = abs(v)**2
                    if f > self._cutoff_frequency:
                        for j, t in enumerate(self._temperatures):
                            dists[j, i] += self.get_Q2(f, t) * (
                                v2[patom1] * c1 +
                                cross_term * c_cross + v2[patom2] * c2)

        self._atom_pairs = atom_pairs
        self._distances = dists / len(self._frequencies)

    def write_yaml(self):
        natom = len(self._masses)
        f = open('thermal_distances.yaml', 'w')
        f.write("natom: %5d\n" % (natom))
        f.write("cutoff_frequency: %f\n" % self._cutoff_frequency)

        f.write("thermal_distances:\n")
        for t, u in zip(self._temperatures, self._distances):
            f.write("- temperature:   %15.7f\n" % t)
            f.write("  distance:\n")
            for i, (atom1, atom2) in enumerate(self._atom_pairs):
                f.write("  - %10.7f # atom pair %d-%d\n"
                        % (u[i], atom1 + 1, atom2 + 1))




# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Thermal displacement plot (tdplot)
#
# Usage:
#   tdplot -i "1 2, 4 5" -o "td.pdf"
#
# The axis resolved thermal displacements are averaged by root mean
# square with the successive indices separated by ",". In this
# example, values at a temperature with the indices 1 and 2, 3 and 4
# are averaged by root mean square respectively as follwos:
#   sqrt( ( x_1 ** 2 + x_2 ** 2 + x_3 ** 2 + x_4 ** 2 ) / 4 ),
# and then they are ploted as a function of temperature.
#
# The definition is that indices correspond to those as follows:
# 1 2 3 : X Y Z of the 1st atom,
# 4 5 6 : X Y Z of the 2nd atom,
# ...

import sys
import numpy as np
import matplotlib.pyplot as plt

try:
    import yaml
except ImportError:
    print("You need to install python-yaml.")
    sys.exit(1)
    
try:
    from yaml import CLoader as Loader
    from yaml import CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

# Parse options
from optparse import OptionParser
parser = OptionParser()
parser.set_defaults(output_filename=None,
                    factor=1.0,
                    td_indices=None,
                    show_legend=False,
                    is_distance=False,
                    is_gnuplot=False,
                    ymax=None,
                    ymin=None,
                    tmax=None,
                    tmin=None)
parser.add_option("-i", "--indices", dest="td_indices",
                  action="store", type="string",
                  help="Indices like 1 2, 3 4 5 6...")
parser.add_option("--factor", dest="factor", type="float",
                  help="Conversion factor of energy unit to internal electronic energy")
parser.add_option("--ymax", dest="ymax", type="float",
                  help="Maximum value of y axis")
parser.add_option("--ymin", dest="ymin", type="float",
                  help="Minimum value of y axis")
parser.add_option("--tmax", dest="tmax", type="float",
                  help="Maximum value of temperature")
parser.add_option("--tmin", dest="tmin", type="float",
                  help="Minimum value of temperature")
parser.add_option("-o", "--output", dest="output_filename",
                  action="store", type="string",
                  help="Output filename")
parser.add_option("-l", "--legend", dest="show_legend",
                  action="store_true",
                  help="Show legend")
parser.add_option("--distance", dest="is_distance",
                  action="store_true",
                  help="Plot thermal distance")
(options, args) = parser.parse_args()


if not options.is_distance:
    if len(args) == 0:
        filename = 'thermal_displacements.yaml'
    else:
        filename = args[0]
    yamldata = yaml.load(open(filename).read(), Loader=Loader)

    td = yamldata['thermal_displacements']
    temperatures = [v['temperature'] for v in td]
    displacements = [v['displacements'] for v in td]
    displacements = np.array(displacements).reshape(len(temperatures), -1)
else:
    if len(args) == 0:
        filename = 'thermal_distances.yaml'
    else:
        filename = args[0]
    yamldata = yaml.load(open(filename).read(), Loader=Loader)

    td = yamldata['thermal_distances']
    temperatures = [v['temperature'] for v in td]
    distances = [v['distance'] for v in td]
    for t, dists in zip(temperatures, distances):
        print(("%14.7f" * (len(dists) + 1)) % ((t,) + tuple(dists)))
        print('')
    print('')
    print('')
    sys.exit(0)

        

# Set temperature range
tmin_index = 0
tmax_index = len(temperatures)
if options.tmin is not None:
    for i, t in enumerate(temperatures):
        if t > options.tmin - (temperatures[1] - temperatures[0]) * 0.1:
            tmin_index = i
            break

if options.tmax is not None:
    for i, t in enumerate(temperatures):
        if t > options.tmax + (temperatures[1] - temperatures[0]) * 0.1:
            tmax_index = i
            break

# Extract indices
set_of_indices = []
if options.td_indices is None:
    set_of_indices.append(range(1, displacements.shape[1] + 1))
else:
    for v in options.td_indices.split(','):
        set_of_indices.append([int(x) for x in v.split()])

print(set_of_indices)

# Plot
plots = []
for indices in set_of_indices:
    d = np.zeros(len(temperatures), dtype='double')
    for i in indices:
        d += displacements[:, i-1]
    plots.append(plt.plot(temperatures[tmin_index:tmax_index],
                          d[tmin_index:tmax_index] / len(indices)))

# Set y range
if (options.ymin is not None) and (options.ymax is not None):
    plt.ylim(options.ymin, options.ymax)
elif (options.ymin is None) and (options.ymax is not None):
    plt.ylim(ymax=options.ymax)
elif (options.ymin is not None) and (options.ymax is None):
    plt.ylim(ymin=options.ymin)

if options.show_legend:
    plt.legend(plots, set_of_indices, loc='upper left')

# Set output device and show
if options.output_filename is not None:
    plt.rcParams['pdf.fonttype'] = 42
    plt.rcParams['font.family'] = 'serif'
    plt.savefig(options.output_filename)
else:
    plt.show()



# Copyright (C) 2013 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from phonopy.structure.cells import get_reduced_bases

search_space = np.array([
        [0, 0, 0],
        [0, 0, 1],
        [0, 1, -1],
        [0, 1, 0],
        [0, 1, 1],
        [1, -1, -1],
        [1, -1, 0],
        [1, -1, 1],
        [1, 0, -1],
        [1, 0, 0],
        [1, 0, 1],
        [1, 1, -1],
        [1, 1, 0],
        [1, 1, 1],
        [-1, -1, -1],
        [-1, -1, 0],
        [-1, -1, 1],
        [-1, 0, -1],
        [-1, 0, 0],
        [-1, 0, 1],
        [-1, 1, -1],
        [-1, 1, 0],
        [-1, 1, 1],
        [0, -1, -1],
        [0, -1, 0],
        [0, -1, 1],
        [0, 0, -1]], dtype='intc')

def get_qpoints_in_Brillouin_zone(primitive_vectors, qpoints):
    bz = BrillouinZone(primitive_vectors)
    bz.run(qpoints)
    return bz.get_shortest_qpoints()

class BrillouinZone(object):
    def __init__(self, primitive_vectors):
        self._primitive_vectors = primitive_vectors # column vectors
        self._tolerance = min(np.sum(primitive_vectors ** 2, axis=0)) * 0.01
        self._reduced_bases = get_reduced_bases(primitive_vectors.T)
        self._tmat = np.dot(np.linalg.inv(self._primitive_vectors),
                            self._reduced_bases)
        self._tmat_inv = np.linalg.inv(self._tmat)
        self._shortest_qpoints = None

    def run(self, qpoints):
        reduced_qpoints = np.dot(qpoints, self._tmat_inv.T)
        self._shortest_qpoints = []
        for q in reduced_qpoints:
            distances = np.array([(np.dot(self._reduced_bases, q + g) ** 2).sum()
                                  for g in search_space], dtype='double')
            min_dist = min(distances)
            shortest_indices = [i for i, d in enumerate(distances - min_dist)
                                if abs(d) < self._tolerance]
            self._shortest_qpoints.append(
                np.dot(search_space[shortest_indices] + q, self._tmat.T))

    def get_shortest_qpoints(self):
        return self._shortest_qpoints

if __name__ == '__main__':
    from phonopy.interface.vasp import read_vasp
    from phonopy.structure.symmetry import Symmetry, get_lattice_vector_equivalence
    from phonopy.structure.spglib import get_ir_reciprocal_mesh, relocate_BZ_grid_address
    import sys

    cell = read_vasp(sys.argv[1])
    symmetry = Symmetry(cell)
    mesh = [4, 4, 4]
    is_shift = np.array([0, 0, 0], dtype='intc')
    mapping_table, grid_address = get_ir_reciprocal_mesh(
        mesh,
        cell,
        is_shift=is_shift)
    ir_grid_points = np.unique(mapping_table)
    primitive_vectors = np.linalg.inv(cell.get_cell())
    bz_grid_address, bz_map = relocate_BZ_grid_address(
        grid_address,
        mesh,
        np.linalg.inv(cell.get_cell()),
        is_shift=is_shift)

    bz_points = np.extract(bz_map > -1, bz_map)
    qpoints = (grid_address + is_shift / 2.0) / mesh
    qpoints -= (qpoints > 0.5001) * 1

    bz = BrillouinZone(primitive_vectors)
    bz.run(qpoints)
    sv = bz.get_shortest_qpoints()
    print("%d %d" % (len(bz_points), np.sum(len(x) for x in sv)))
    for q, vs in zip(qpoints, sv):
        if np.allclose(q, vs[0]):
            print(q)
        else:
            print("%s * %s" % (q, np.linalg.norm(np.dot(primitive_vectors, q))))
        for v in vs:
            print("%s %s" % (v, np.linalg.norm(np.dot(primitive_vectors, v))))

    rotations = symmetry.get_reciprocal_operations()
    print(get_lattice_vector_equivalence(rotations))



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np

directions_axis = np.array([[ 1, 0, 0 ],
                            [ 0, 1, 0 ],
                            [ 0, 0, 1 ]])

directions_diag = np.array([[ 1, 0, 0 ],
                            [ 0, 1, 0 ],
                            [ 0, 0, 1 ],
                            [ 1, 1, 0 ],
                            [ 1, 0, 1 ],
                            [ 0, 1, 1 ],
                            [ 1,-1, 0 ],
                            [ 1, 0,-1 ],
                            [ 0, 1,-1 ],
                            [ 1, 1, 1 ],
                            [ 1, 1,-1 ],
                            [ 1,-1, 1 ],
                            [-1, 1, 1 ]])

def direction_to_displacement(displacement_directions,
                              distance,
                              supercell):
    lattice = supercell.get_cell()
    first_atoms = []
    for disp in displacement_directions:
        direction = disp[1:]
        disp_cartesian = np.dot(direction, lattice)
        disp_cartesian *= distance / np.linalg.norm(disp_cartesian)
        first_atoms.append({'number': disp[0],
                            'displacement': disp_cartesian,
                            'direction': direction})
    displacement_dataset = {
        'natom': supercell.get_number_of_atoms(),
        'first_atoms': first_atoms}
    
    return displacement_dataset

def get_least_displacements(symmetry,
                            is_plusminus='auto',
                            is_diagonal=True,
                            is_trigonal=False,
                            log_level=0):
    """
    Return least displacements

    Format:
      [[atom_num, disp_x, disp_y, disp_z],
       [atom_num, disp_x, disp_y, disp_z],
       ...
     ]
    """
    displacements = []
    if is_diagonal:
        directions = directions_diag
    else:
        directions = directions_axis

    if log_level > 2:
        print("Site point symmetry:")

    for atom_num in symmetry.get_independent_atoms():
        site_symmetry = symmetry.get_site_symmetry(atom_num)

        if log_level > 2:
            print("Atom %d" % (atom_num + 1))
            for i, rot in enumerate(site_symmetry):
                print("----%d----" % (i + 1))
                for v in rot:
                    print("%2d %2d %2d" % tuple(v))
        
        for disp in get_displacement(site_symmetry,
                                     directions,
                                     is_trigonal,
                                     log_level):
            displacements.append([atom_num,
                                  disp[0], disp[1], disp[2]])
            if is_plusminus == 'auto':
                if is_minus_displacement(disp, site_symmetry):
                    displacements.append([atom_num,
                                          -disp[0], -disp[1], -disp[2]])
            elif is_plusminus is True:
                displacements.append([atom_num,
                                      -disp[0], -disp[1], -disp[2]])

    return displacements

def get_displacement(site_symmetry,
                     directions=directions_diag,
                     is_trigonal=False,
                     log_level=0):
    # One
    sitesym_num, disp = get_displacement_one(site_symmetry,
                                             directions)
    if disp is not None:
        if log_level > 2:
            print("Site symmetry used to expand a direction %s" % disp[0])
            print(site_symmetry[sitesym_num])
        return disp
    # Two
    sitesym_num, disps = get_displacement_two(site_symmetry,
                                              directions)
    if disps is not None:
        if log_level > 2:
            print("Site symmetry used to expand directions %s %s" %
                  (disps[0], disps[1]))
            print(site_symmetry[sitesym_num])

        if is_trigonal:
            disps_new = [disps[0]]
            if is_trigonal_axis(site_symmetry[sitesym_num]):
                if log_level > 2:
                    print("Trigonal axis is found.")
                disps_new.append(np.dot(disps[0],
                                        site_symmetry[sitesym_num].T))
                disps_new.append(np.dot(disps_new[1],
                                        site_symmetry[sitesym_num].T))
            disps_new.append(disps[1])
            return disps_new
        else:
            return disps
    # Three
    return [directions[0], directions[1], directions[2]]

def get_displacement_one(site_symmetry,
                         directions=directions_diag):
    for direction in directions:
        rot_directions = []
        for r in site_symmetry:
            rot_directions.append(np.dot(direction, r.T))
        num_sitesym = len(site_symmetry)
        for i in range(num_sitesym):
            for j in range(i+1, num_sitesym):
                det = determinant(direction,
                                  rot_directions[i],
                                  rot_directions[j])
                if det != 0:
                    return i, [direction]
    return None, None

def get_displacement_two(site_symmetry,
                         directions=directions_diag):
    for direction in directions:
        rot_directions = []
        for r in site_symmetry:
            rot_directions.append(np.dot(direction, r.T))
        num_sitesym = len(site_symmetry)
        for i in range(num_sitesym):
            for second_direction in directions:
                det = determinant(direction,
                                  rot_directions[i],
                                  second_direction)
                if det != 0:
                    return i, [direction, second_direction]
    return None, None
    
def is_minus_displacement(direction, site_symmetry):
    is_minus = True
    for r in site_symmetry:
        rot_direction = np.dot(direction, r.T)
        if (rot_direction + direction).any():
            continue
        else:
            is_minus = False
            break
    return is_minus

def is_trigonal_axis(r):
    r3 = np.dot(np.dot(r, r), r)
    if (r3 == np.eye(3, dtype=int)).all():
        return True
    else:
        return False

def determinant(a, b, c):
    det = a[0] * b[1] * c[2] - a[0] * b[2] * c[1] \
        + a[1] * b[2] * c[0] - a[1] * b[0] * c[2] \
        + a[2] * b[0] * c[1] - a[2] * b[1] * c[0]
    return det
        
        
def print_displacements(symmetry,
                        directions=directions_diag):
    displacements = get_least_displacements(symmetry, directions)
    print("Least displacements:")
    print("Atom       Directions")
    print("----------------------------")
    for key in displacements:
        print("%4d  %s" % (key + 1, displacements[key]))





# Copyright (C) 2012 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
from phonopy import Phonopy
from phonopy import PhonopyGruneisen
from phonopy.interface import read_crystal_structure, get_default_cell_filename
from phonopy.file_IO import (parse_FORCE_SETS, parse_BORN,
                             parse_FORCE_CONSTANTS, read_force_constants_hdf5)
from phonopy.units import VaspToTHz, Wien2kToTHz, AbinitToTHz, PwscfToTHz
import numpy as np

def fracval(frac):
    if frac.find('/') == -1:
        return float(frac)
    else:
        x = frac.split('/')
        return float(x[0]) / float(x[1])

def get_cell(cell_filename,
             interface_mode):
    cell, optional_structure_file_information = read_crystal_structure(
        filename=cell_filename,
        interface_mode=interface_mode,
        yaml_mode=options.yaml_mode)
    if cell is None:
        print("Crystal structure file of %s could not be found." %
              optional_structure_file_information[0])
        sys.exit(1)

    return cell
        
def get_phonon(cell_filename,
               force_sets_filename,
               dim,
               primitive_matrix=np.eye(3),
               born_filename=None,
               factor=VaspToTHz,
               interface_mode='vasp',
               symprec=1e-5):
    cell = get_cell(cell_filename, interface_mode)
    phonon = Phonopy(cell,
                     dim,
                     primitive_matrix=primitive_matrix,
                     factor=factor,
                     symprec=symprec)
    force_sets = parse_FORCE_SETS(filename=force_sets_filename)
    phonon.set_displacement_dataset(force_sets)
    phonon.produce_force_constants()
    if born_filename:
        nac_params = parse_BORN(phonon.get_primitive(),
                                filename=born_filename,
                                symprec=symprec)
        phonon.set_nac_params(nac_params)
    phonon.set_dynamical_matrix()
    return phonon

def get_phonon_from_force_constants(cell_filename,
                                    force_constants_filename,
                                    dim,
                                    primitive_matrix=np.eye(3),
                                    born_filename=None,
                                    factor=VaspToTHz,
                                    interface_mode='vasp',
                                    symprec=1e-5,
                                    is_hdf5=False):
    cell = get_cell(cell_filename, interface_mode)
    phonon = Phonopy(cell,
                     dim,
                     primitive_matrix=primitive_matrix,
                     factor=factor,
                     symprec=symprec)
    if is_hdf5:
        force_constants = read_force_constants_hdf5(
            filename=force_constants_filename)
    else:
        force_constants = parse_FORCE_CONSTANTS(
            filename=force_constants_filename)
    phonon.set_force_constants(force_constants)
    if born_filename:
        nac_params = parse_BORN(phonon.get_primitive(),
                                filename=born_filename,
                                symprec=symprec)
        phonon.set_nac_params(nac_params)
    phonon.set_dynamical_matrix()
    return phonon

from optparse import OptionParser
parser = OptionParser()
parser.set_defaults(
    abinit_mode=False,
    band_paths=None,
    cell_filename=None,
    color_scheme=None,
    cutoff_frequency=None,
    cutoff_wave_vector=1e-4,
    factor=None,
    is_gamma_center=False,
    is_hdf5=False,
    is_nac=False,
    is_mesh_symmetry=True,
    plot_graph=False,
    reads_force_constants=False,                    
    band_points=51,
    marker='o',
    markersize=None,
    primitive_axis=None,
    pwscf_mode=False,
    sampling_mesh=None,
    save_graph=False,
    supercell_dimension=None,
    symprec=1e-5,
    title=None,
    tmax=2004,
    tmin=0,
    tstep=2,
    volumes_filename=None,
    wien2k_mode=False,
    yaml_mode=False)
parser.add_option("--abinit", dest="abinit_mode",
                  action="store_true", help="Invoke Abinit mode")
parser.add_option("--band", dest="band_paths",
                  action="store", type="string",
                  help="Band paths in reduced coordinates")
parser.add_option("--band_points", dest="band_points", type="int",
                   help="Number of sampling points in a segment of band path")
parser.add_option("-c", "--cell", dest="cell_filename",
                  action="store", type="string",
                  help="Read unit cell", metavar="FILE")
parser.add_option("--color", dest="color_scheme",
                  action="store", type="string",
                  help="Color scheme")
parser.add_option("--cutoff", dest="cutoff_frequency", type="float",
                  help="Plot above this cutoff frequency for mesh sampling mode.")
parser.add_option("--dim", dest="supercell_dimension",
                  action="store", type="string",
                  help="Same behavior as DIM tag")
parser.add_option("--factor", dest="factor", type="float",
                  help="Conversion factor to favorite frequency unit")
parser.add_option("--hdf5", dest="is_hdf5", action="store_true",
                  help="Use hdf5 to read force constants and store results")
parser.add_option("--gc", "--gamma_center", dest="is_gamma_center",
                  action="store_true",
                  help="Set mesh as Gamma center")
parser.add_option("--marker", dest="marker",
                  action="store", type="string",
                  help="Marker for plot (matplotlib)")
parser.add_option("--markersize", dest="markersize", type="float",
                  help="Markersize for plot in points (matplotlib)")
parser.add_option("--mp", "--mesh", dest="sampling_mesh",
                  action="store", type="string",
                  help="Sampling mesh")
parser.add_option("--nac", dest="is_nac",
                  action="store_true",
                  help="Non-analytical term correction")
parser.add_option("--nomeshsym", dest="is_mesh_symmetry",
                  action="store_false",
                  help="Symmetry is not imposed for mesh sampling.")
parser.add_option("-o", "--output", dest="output_filename",
                  action="store", type="string",
                  help="Output filename of PDF plot")
parser.add_option("-p", "--plot", dest="plot_graph",
                  action="store_true",
                  help="Plot data")
parser.add_option("--pa", "--primitive_axis", dest="primitive_axis",
                  action="store", type="string",
                  help="Same as PRIMITIVE_AXIS tags")
parser.add_option("--pwscf", dest="pwscf_mode",
                  action="store_true", help="Invoke Pwscf mode")
parser.add_option("--q_cutoff", dest="cutoff_wave_vector", type="float",
                  help="Acoustic modes inside cutoff wave vector is treated.")
parser.add_option("--readfc", dest="reads_force_constants",
                  action="store_true",
                  help="Read FORCE_CONSTANTS")
parser.add_option("-s", "--save", dest="save_graph",
                  action="store_true",
                  help="Save plot data in pdf")
parser.add_option("-t", "--title", dest="title", action="store",
                  type="string", help="Title of plot")
parser.add_option("--tmax", dest="tmax", type="float",
                  help="Maximum calculated temperature")
parser.add_option("--tmin", dest="tmin", type="float",
                  help="Minimum calculated temperature")
parser.add_option("--tolerance", dest="symprec", type="float",
                  help="Symmetry tolerance to search")
parser.add_option("--tstep", dest="tstep", type="float",
                  help="Calculated temperature step")
parser.add_option("--vf", "--volumes_filename", dest="volumes_filename",
                  action="store", type="string",
                  help="Filename of volume is contained.")
parser.add_option("--wien2k", dest="wien2k_mode",
                  action="store_true", help="Invoke Wien2k mode")
parser.add_option("--yaml", dest="yaml_mode",
                  action="store_true", help="Activate phonopy YAML mode")
(options, args) = parser.parse_args()

if options.is_hdf5:
    try:
        import h5py
    except ImportError:
        print_error_message("You need to install python-h5py.")
        if log_level > 0:
            print_end()
        sys.exit(1)

if len(args) is not 3:
    sys.stderr.write("Three directory names (original, plus, minus) "
                     "have to be spefied.\n")
    sys.exit(1)

if options.primitive_axis:
    primitive_axis = np.array(
        [fracval(x) for x in options.primitive_axis.split()]).reshape(3, 3)
else:
    primitive_axis = np.eye(3)

if options.supercell_dimension:
    dim = [int(x) for x in options.supercell_dimension.split()]
    if len(dim) == 9:
        dim = np.array(dim).reshape(3, 3)
    elif len(dim) == 3:
        dim = np.diag(dim)
    else:
        print("Number of elements of --dim option has to be 3 or 9.")
        sys.exit(1)
    if np.linalg.det(dim) < 1:
        print("Determinant of supercell matrix has to be positive.")
        sys.exit(1)
else:
    print("--dim option has to be specified.")
    sys.exit(1)


#
# Phonopy interface mode
#
# Physical units: energy,  distance,  atomic mass, force
# vasp          : eV,      Angstrom,  AMU,         eV/Angstrom
# wien2k        : Ry,      au,        AMU,         mRy/au
# pwscf         : Ry,      au,        AMU,         Ry/au
# abinit        : hartree, au(=bohr), AMU,         eV/Angstrom
#
if options.wien2k_mode:
    interface_mode = 'wien2k'
    factor = Wien2kToTHz
elif options.abinit_mode:
    interface_mode = 'abinit'
    factor = AbinitToTHz
elif options.pwscf_mode:
    interface_mode = 'pwscf'
    factor = PwscfToTHz
else:
    interface_mode = 'vasp'
    factor = VaspToTHz

if options.factor is not None:
    factor = options.factor
    
if options.cell_filename:
    cell_filename = options.cell_filename
else:
    cell_filename = get_default_cell_filename(interface_mode)
    
    
phonons = []
for i in range(3):
    directory = args[i]
    if options.is_nac:
        born_filename = "%s/BORN" % directory
    else:
        born_filename = None
    
    if options.reads_force_constants:
        if options.is_hdf5:
            phonon = get_phonon_from_force_constants(
                "%s/%s" % (directory, cell_filename),
                "%s/force_constants.hdf5" % directory,
                dim,
                primitive_matrix=primitive_axis,
                born_filename=born_filename,
                factor=factor,
                interface_mode=interface_mode,
                symprec=options.symprec,
                is_hdf5=True)
        else:
            phonon = get_phonon_from_force_constants(
                "%s/%s" % (directory, cell_filename),
                "%s/FORCE_CONSTANTS" % directory,
                dim,
                primitive_matrix=primitive_axis,
                born_filename=born_filename,
                factor=factor,
                interface_mode=interface_mode,
                symprec=options.symprec)
        phonons.append(phonon)
    else:
        phonon = get_phonon("%s/%s" % (directory, cell_filename),
                            "%s/FORCE_SETS" % directory,
                            dim,
                            primitive_matrix=primitive_axis,
                            born_filename=born_filename,
                            factor=factor,
                            interface_mode=interface_mode,
                            symprec=options.symprec)
        phonons.append(phonon)

gruneisen = PhonopyGruneisen(phonons[0], # equilibrium
                             phonons[1], # plus
                             phonons[2]) # minus

if options.plot_graph:
    if options.save_graph:
        import matplotlib as mpl
        mpl.use('Agg')
        
if options.band_paths:
    band_paths = []
    for path_str in options.band_paths.split(','):
        paths = np.array(
            [fracval(x) for x in path_str.split()]).reshape(-1, 3)
        for i in range(len(paths) - 1):
            band_paths.append([paths[i], paths[i + 1]])

    gruneisen.set_band_structure(band_paths,
                                 options.band_points)
    gruneisen.write_yaml_band_structure()
    if options.plot_graph:
        plt = gruneisen.plot_band_structure(epsilon=options.cutoff_wave_vector,
                                            color_scheme=options.color_scheme)
        if options.title is not None:
            plt.suptitle(options.title)


elif options.sampling_mesh:
    mesh_numbers = np.array([int(x) for x in options.sampling_mesh.split()])
    gruneisen.set_mesh(mesh_numbers,
                       is_gamma_center=options.is_gamma_center,
                       is_mesh_symmetry=options.is_mesh_symmetry)

    if options.is_hdf5:
        gruneisen.write_hdf5_mesh()
    else:
        gruneisen.write_yaml_mesh()
    
    if options.plot_graph:
        plt = gruneisen.plot_mesh(cutoff_frequency=options.cutoff_frequency,
                                  color_scheme=options.color_scheme,
                                  marker=options.marker,
                                  markersize=options.markersize)
        if options.title is not None:
            plt.suptitle(options.title)

    if options.volumes_filename is not None:
        volumes = []
        with open(options.volumes_filename) as f:
            for line in f:
                if line.strip()[0] == '#':
                    continue
                volumes.append(float(line.split()[0]))
        if volumes:
            gruneisen.set_thermal_properties(
                volumes,
                t_step=options.tstep,
                t_max=options.tmax,
                t_min=options.tmin,
                cutoff_frequency=options.cutoff_frequency)
            gruneisen.write_yaml_thermal_properties()
else:
    pass

if options.plot_graph:
    if options.save_graph:
        if options.output_filename:
            plt.savefig(options.output_filename)
        else:
            plt.savefig("gruneisen.pdf")
    else:
        plt.show()




# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Thermal properties plot (propplot)
#
# Usage:
#   propplot

import numpy as np

try:
    import yaml
except ImportError:
    print("You need to install python-yaml.")
    sys.exit(1)
    
try:
    from yaml import CLoader as Loader
    from yaml import CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

# Parse options
from optparse import OptionParser
parser = OptionParser()
parser.set_defaults( output_filename = None,
                     factor = 1.0,
                     is_heat_capacity = False,
                     is_entropy = False,
                     is_free_energy = False,
                     is_diff = False,
                     is_gnuplot = False,
                     ymax = None,
                     ymin = None,
                     tmax = None,
                     tmin = None )
parser.add_option("--factor", dest="factor", type="float",
                  help="Conversion factor of energy unit to internal electronic energy")
parser.add_option("--ymax", dest="ymax", type="float",
                  help="Maximum value of y axis")
parser.add_option("--ymin", dest="ymin", type="float",
                  help="Minimum value of y axis")
parser.add_option("--tmax", dest="tmax", type="float",
                  help="Maximum value of temperature")
parser.add_option("--tmin", dest="tmin", type="float",
                  help="Minimum value of temperature")
parser.add_option("--cv", "--heat_capacity", dest="is_heat_capacity",
                  action="store_true",
                  help="Plot heat capacity")
parser.add_option("-s", "--entropy", dest="is_entropy",
                  action="store_true",
                  help="Plot entropy")
parser.add_option("--fe", "--free_energy", dest="is_free_energy",
                  action="store_true",
                  help="Plot free energy")
parser.add_option("-o", "--output", dest="output_filename",
                  action="store", type="string",
                  help="Output filename")
parser.add_option("-d", "--diff", dest="is_diff",
                  action="store_true",
                  help="Calculate difference")
parser.add_option("--gnuplot", dest="is_gnuplot",
                  action="store_true",
                  help="Damp thermal properties in gnuplot data style")
(options, args) = parser.parse_args()

if options.is_gnuplot:
    print("# temperature[K] Free energy [kJ/mol] Entropy [J/K/mol] "
          "Heat capacity [J/K/mol] Energy [kJ/mol]")
else:
    import matplotlib.pyplot as plt
    if ( ( not options.is_heat_capacity ) and 
         ( not options.is_entropy ) and 
         ( not options.is_free_energy ) ):
        print("Set --cv, --entropy or --fe")
        sys.exit(0)

if len( args ) == 0:
    filenames = ['thermal_properties.yaml']
else:
    filenames = args

if options.is_heat_capacity:
    prop_target = 'heat_capacity'
elif options.is_entropy:
    prop_target = 'entropy'
elif options.is_free_energy:
    prop_target = 'free_energy'

thermal_properties_0 = yaml.load(open(filenames[0]).read(),
                                 Loader=Loader)['thermal_properties']
temperatures = [v['temperature'] for v in thermal_properties_0]

tmin_index = 0
tmax_index = len(temperatures)
if options.tmin is not None:
    for i, t in enumerate(temperatures):
        if t > options.tmin - (temperatures[1] - temperatures[0]) * 0.1:
            tmin_index = i
            break

if options.tmax is not None:
    for i, t in enumerate(temperatures):
        if t > options.tmax + (temperatures[1] - temperatures[0]) * 0.1:
            tmax_index = i
            break

if options.is_diff:
    props_0 = [v[prop_target] for v in thermal_properties_0]
    props_0 = np.array(props_0) * options.factor

for filename in filenames:
    thermal_properties = yaml.load(open(filename).read(),
                                   Loader=Loader)['thermal_properties']

    if options.is_gnuplot:
        props = []
        for name in ('temperature', 'free_energy', 'entropy', 'heat_capacity'):
            props.append([v[name] for v in thermal_properties])
            
        for t, f, e, h in zip(props[0][tmin_index:tmax_index],
                              props[1][tmin_index:tmax_index],
                              props[2][tmin_index:tmax_index],
                              props[3][tmin_index:tmax_index]):
            print(("%14.7f " * 5) % (t, f, e, h, f + e * t / 1000))
        print('')
        print('')
    else:
        temperatures = [v['temperature'] for v in thermal_properties]
        props = [v[prop_target] for v in thermal_properties]
        props = np.array(props) * options.factor
        if options.is_diff:
            props -= props_0
        plt.plot(temperatures[tmin_index:tmax_index],
                 props[tmin_index:tmax_index])

        

if not options.is_gnuplot:
    if (options.ymin is not None) and (options.ymax is not None):
        plt.ylim(options.ymin, options.ymax)
    elif (options.ymin is None) and (options.ymax is not None):
        plt.ylim(ymax=options.ymax)
    elif (options.ymin is not None) and (options.ymax is None):
        plt.ylim(ymin=options.ymin)
    
    if options.output_filename is not None:
        plt.rcParams['pdf.fonttype'] = 42
        plt.rcParams['font.family'] = 'serif'
        plt.savefig(options.output_filename)
    else:
        plt.show()



# Copyright (C) 2015 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from phonopy.qha import *
from phonopy.units import EvTokJmol, EVAngstromToGPa

class PhonopyQHA(object):
    def __init__(self,
                 volumes,
                 electronic_energies,
                 eos='vinet',
                 temperatures=None,
                 free_energy=None,
                 cv=None,
                 entropy=None,
                 t_max=None,
                 verbose=False):
        """
        The following two have the same number of elements
          volumes: Unit cell volumes (V) in Angstrom^3
          electronic_energies: Electronic energies (U) in eV

        The following four have the same number of elements
          temperatures: Temperatures ascending order (T) in K
          cv: Heat capacity at constant volume in J/K/mol
          entropy: Entropy at constant volume (S) J/K/mol
          free_energy: Helmholtz free energy (F) kJ/mol

        eos: Equation of state used for fitting F vs V
             'vinet', 'murnaghan' or 'birch_murnaghan'
        tmax: Maximum temperature to be calculated. This has to be not
              greater than the temperature of the third element from the
              end of 'temperatre' elements. If max_t=None, the temperature
              of the third element from the end is used.
        """
        self._bulk_modulus = BulkModulus(volumes,
                                         electronic_energies,
                                         eos=eos)

        if temperatures is not None:
            self._qha = QHA(volumes,
                            electronic_energies,
                            temperatures,
                            cv,
                            entropy,
                            free_energy,
                            eos=eos,
                            t_max=t_max)
            self._qha.run(verbose=verbose)

    def get_bulk_modulus(self):
        return self._bulk_modulus.get_bulk_modulus()
            
    def get_bulk_modulus_parameters(self):
        """Returns bulk modulus
        (lowest energy,
         bulk modulus,
         b_prime,
         equilibrium volume)
        """
        return self._bulk_modulus.get_parameters()
    
    def plot_bulk_modulus(self):
        return self._bulk_modulus.plot()

    def get_eos(self):
        return self._qha.get_eos()

    def plot_qha(self, thin_number=10, volume_temp_exp=None):
        return self._qha.plot(thin_number=thin_number,
                              volume_temp_exp=volume_temp_exp)

    def get_helmholtz_volume(self):
        """Returns free_energies
        
        free_energies: Free energies calculated at temperatures and volumes
                       [temperatures][volumes]
        """
        return self._qha.get_helmholtz_volume()

    def plot_helmholtz_volume(self,
                              thin_number=10,
                              plt=None,
                              xlabel=r'Volume $(\AA^3)$',
                              ylabel='Free energy'):
        return self._qha.plot_helmholtz_volume(thin_number=thin_number,
                                               plt=plt,
                                               xlabel=xlabel,
                                               ylabel=ylabel)

    def plot_pdf_helmholtz_volume(self,
                                  thin_number=10,
                                  filename='helmholtz-volume.pdf'):
        self._qha.plot_pdf_helmholtz_volume(thin_number=thin_number,
                                            filename=filename)

    def write_helmholtz_volume(self, filename='helmholtz-volume.dat'):
        self._qha.write_helmholtz_volume(filename=filename)

    def get_volume_temperature(self):
        """Returns volumes at temperatures"""
        return self._qha.get_volume_temperature()

    def plot_volume_temperature(self, exp_data=None, plt=None):
        return self._qha.plot_volume_temperature(exp_data=exp_data, plt=plt)

    def plot_pdf_volume_temperature(self,
                                    exp_data=None,
                                    filename='volume-temperature.pdf'):
        self._qha.plot_pdf_volume_temperature(exp_data=exp_data,
                                              filename=filename)

    def write_volume_temperature(self, filename='volume-temperature.dat'):
        self._qha.write_volume_temperature(filename=filename)

    def get_thermal_expansion(self):
        """Returns thermal expansion coefficients at temperatures"""
        return self._qha.get_thermal_expansion()

    def plot_thermal_expansion(self, plt=None):
        return self._qha.plot_thermal_expansion(plt=plt)

    def plot_pdf_thermal_expansion(self,
                                   filename='thermal_expansion.pdf'):
        self._qha.plot_pdf_thermal_expansion(filename=filename)

    def write_thermal_expansion(self,
                                filename='thermal_expansion.dat'):
        self._qha.write_thermal_expansion(filename=filename)

    def get_volume_expansion(self):
        """Return volume expansions at temperatures"""
        return self._qha.get_volume_expansion()

    def plot_volume_expansion(self, exp_data=None, symbol='o', plt=None):
        return self._qha.plot_volume_expansion(exp_data=exp_data,
                                               symbol=symbol,
                                               plt=plt)

    def plot_pdf_volume_expansion(self,
                                  exp_data=None,
                                  symbol='o',
                                  filename='volume_expansion.pdf'):
        self._qha.plot_pdf_volume_expansion(exp_data=exp_data,
                                            symbol=symbol,
                                            filename=filename)

    def write_volume_expansion(self, filename='volume_expansion.dat'):
        self._qha.write_volume_expansion(filename=filename)

    def get_gibbs_temperature(self):
        """Returns Gibbs free energies at temperatures"""
        return self._qha.get_gibbs_temperature()

    def plot_gibbs_temperature(self,
                               plt=None,
                               xlabel='Temperature (K)',
                               ylabel='Gibbs free energy'):
        return self._qha.plot_gibbs_temperature(plt=plt,
                                                xlabel=xlabel,
                                                ylabel=ylabel)

    def plot_pdf_gibbs_temperature(self, filename='gibbs-temperature.pdf'):
        self._qha.plot_pdf_gibbs_temperature(filename=filename)

    def write_gibbs_temperature(self, filename='gibbs-temperature.dat'):
        self._qha.write_gibbs_temperature(filename=filename)

    def get_bulk_modulus_temperature(self):
        """Returns bulk modulus at temperatures"""
        return self._qha.get_bulk_modulus_temperature()

    def plot_bulk_modulus_temperature(self,
                                      plt=None,
                                      xlabel='Temperature (K)',
                                      ylabel='Bulk modulus'):
        return self._qha.plot_bulk_modulus_temperature(plt=plt,
                                                       xlabel=xlabel,
                                                       ylabel=ylabel)

    def plot_pdf_bulk_modulus_temperature(self,
                                          filename='bulk_modulus-temperature.pdf'):
        self._qha.plot_pdf_bulk_modulus_temperature(filename=filename)

    def write_bulk_modulus_temperature(self,
                                       filename='bulk_modulus-temperature.dat'):
        self._qha.write_bulk_modulus_temperature(filename=filename)

    def get_heat_capacity_P_numerical(self):
        """Returns heat capacities at constant pressure at temperatures
        
        These values are calculated by -T*d^2G/dT^2.
        """
        return self._qha.get_heat_capacity_P_numerical()

    def plot_heat_capacity_P_numerical(self, Z=1, exp_data=None, plt=None):
        return self._qha.plot_heat_capacity_P_numerical(Z=Z,
                                                        exp_data=exp_data,
                                                        plt=plt)

    def plot_pdf_heat_capacity_P_numerical(self,
                                           exp_data=None,
                                           filename='Cp-temperature.pdf'):
        self._qha.plot_pdf_heat_capacity_P_numerical(exp_data=exp_data,
                                                     filename=filename)

    def write_heat_capacity_P_numerical(self, filename='Cp-temperature.dat'):
        self._qha.write_heat_capacity_P_numerical(filename=filename)

    def get_heat_capacity_P_polyfit(self):
        """Returns heat capacities at constant pressure at temperatures

        These values are calculated from the values obtained by polynomial
        fittings of Cv and S.
        """
        return self._qha.get_heat_capacity_P_polyfit()

    def plot_heat_capacity_P_polyfit(self, exp_data=None, Z=1, plt=None):
        return self._qha.plot_heat_capacity_P_polyfit(Z=Z,
                                                      exp_data=exp_data,
                                                      plt=plt)

    def plot_pdf_heat_capacity_P_polyfit(self,
                                         exp_data=None,
                                         filename='Cp-temperature_polyfit.pdf'):
        self._qha.plot_pdf_heat_capacity_P_polyfit(exp_data=exp_data,
                                                   filename=filename)

    def write_heat_capacity_P_polyfit(self,
                                      filename='Cp-temperature_polyfit.dat',
                                      filename_ev='entropy-volume.dat',
                                      filename_cvv='Cv-volume.dat',
                                      filename_dsdvt='dsdv-temperature.dat'):
        self._qha.write_heat_capacity_P_polyfit(filename=filename,
                                                filename_ev=filename_ev,
                                                filename_cvv=filename_cvv,
                                                filename_dsdvt=filename_dsdvt)

    def get_gruneisen_temperature(self):
        """Returns Gruneisen parameters at temperatures"""
        return self._qha.get_gruneisen_temperature()

    def plot_gruneisen_temperature(self, plt=None):
        return self._qha.plot_gruneisen_temperature(plt=plt)

    def plot_pdf_gruneisen_temperature(self,
                                       filename='gruneisen-temperature.pdf'):
        self._qha.plot_pdf_gruneisen_temperature(filename=filename)

    def write_gruneisen_temperature(self, filename='gruneisen-temperature.dat'):
        self._qha.write_gruneisen_temperature(filename=filename)
